#ifndef OSL_BRINKMATE_SEARCHER_H
#define OSL_BRINKMATE_SEARCHER_H

#include "osl/brinkmate/brinkmateTable.h"
#include "osl/brinkmate/brinkmateState.h"
#include "osl/brinkmate/threatmatePool.h"
#include "osl/search/bigramKillerMove.h"

namespace osl
{
  namespace checkmate
  {
    class ProofDisproof;
  } // namespace checkmate
  namespace brinkmate
  {
    /**
     * brinkmate searcher with iterative deepening
     */
    class Searcher
    {
      BrinkmateTable table;
      BrinkmateState state;
      BrinkmateState::checkmate_t checkmator;
      Configuration root_conf;
      ThreatmatePool pool;
      search::BigramKillerMove killer_moves;
      volatile int *stop_flag;
      int count;
      int max_depth;
      int verbose;
    public:
      Searcher(Player attack, size_t capacity, int verbose);
      ~Searcher();

      /** iterative deepening, widening 
       * @return move to brinkmate if exists, Move::INVALID() otherwise.
       */
      const Move search(NumEffectState&, int max_depth);
      /** @throw TableFull */
      BrinkmateResult search(NumEffectState&,
			     int depth, int tnode, int cnode,
			     int dcm, int rnode);

      BrinkmateResult attack();
      BrinkmateResult defense();

      void setStopFlag(volatile int *flag)
      {
	stop_flag = flag;
      }

      int depthLeft() const { return root_conf.depth - state.effectiveDepth(); }
      bool provedByRecord(const BrinkmateRecord&) const;
      bool disprovedByRecord(const BrinkmateRecord&) const;
      void showBestMoves() const;
      void showTree(const HashKey&, int depth=1) const;
      int getCount() const { return count; }

      static bool threatmateCandidate(const BrinkmateState&, Move);
      static bool threatmateCandidate(const BrinkmateState&, Move,
				      Position centered_king);
    private:
      /** @return cut Ǥ */
      bool attack(BrinkmateRecord *, Move);
      /** @return cut Ǥ */
      bool defense(BrinkmateRecord *, Move);
      void setUpAttack(BrinkmateRecord *record);
      void setUpDefense(BrinkmateRecord *record);

    };
  } // namespace brinkmate
} // namespace osl

#endif /* OSL_BRINKMATE_SEARCHER_H */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
