/* dualDfpn.h
 */
#ifndef OSL_DUALDFPN_H
#define OSL_DUALDFPN_H
#include "osl/state/numEffectState.h"
#include "osl/hash/hashKey.h"
#include "osl/pathEncoding.h"
#include "osl/container/moveStack.h"
#include <boost/shared_ptr.hpp>
#include <boost/scoped_ptr.hpp>
#include <cstddef>
namespace osl
{
  class RepetitionCounter;
  namespace checkmate
  {
    class DualDfpn
    {
      struct Shared;
      struct Local;
      struct OraclePool;
      boost::shared_ptr<Shared> shared;
      boost::scoped_ptr<Local> local;
    public:
      explicit DualDfpn(size_t limit=(size_t)-1);
      DualDfpn(const DualDfpn& src);
      ~DualDfpn();

      void setRootPlayer(Player);
      /**
       * @return ̤ͤ
       */
      template <Player P>
      bool isWinningState(int node_limit, NumEffectState& state, 
			  const HashKey& key, const PathEncoding& path,
			  Move& best_move, Move last_move=Move::INVALID());
      bool isWinningState(int node_limit, NumEffectState& state, 
			  const HashKey& key, const PathEncoding& path,
			  Move& best_move, Move last_move=Move::INVALID());
      template <Player P>
      bool isLosingState(int node_limit, NumEffectState& state, 
			 const HashKey& key, const PathEncoding& path,
			 Move last_move=Move::INVALID());
      bool isLosingState(int node_limit, NumEffectState& state, 
			 const HashKey& key, const PathEncoding& path,
			 Move last_move=Move::INVALID());

      // debug
      void setVerbose(int level=1);
      int distance(Player attack, const HashKey& key) const;
      size_t mainNodeCount() const;
      size_t totalNodeCount() const;
      void writeRootHistory(const RepetitionCounter& counter,
			    const MoveStack& moves,
			    const SimpleState& state, Player attack);
    private:
      void updateBlockingVerify();
    };
  }
  using checkmate::DualDfpn;
}

#endif /* OSL_DUALDFPN_H */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
