#ifndef _GENERATE_ESCAPE_MOVES_TCC
#define _GENERATE_ESCAPE_MOVES_TCC

#include "osl/move_generator/escape_.h"
#include "osl/move_action/concept.h"
#include "osl/move_action/store.h"
#include "osl/container/moveVector.h"
namespace osl
{
  namespace move_generator
  {
    template<Player P,class Action,bool CheapOnly>
    void Escape<P,Action,CheapOnly>::
    generateDropAll(const NumEffectState& state,Position to,Action& action)
    {
      bool gen = generateDrop<PAWN>(state,to,action); if (CheapOnly && gen) return;
      gen = generateDrop<LANCE>(state,to,action);     if (CheapOnly && gen) return;
      gen = generateDrop<KNIGHT>(state,to,action);    if (CheapOnly && gen) return;
      gen = generateDrop<SILVER>(state,to,action);    if (CheapOnly && gen) return;
      gen = generateDrop<GOLD>(state,to,action);      if (CheapOnly && gen) return;
      gen = generateDrop<BISHOP>(state,to,action);    if (CheapOnly && gen) return;
      generateDrop<ROOK>(state,to,action);      
    }

    template<Player P,class Action,bool CheapOnly>
    void Escape<P,Action,CheapOnly>::
    generateBlockingKing(const NumEffectState& state,Piece p,Position from,Action &action,move_action::NotKingOpenFilter<P,Action>& safe_action)
    {
      Position to=p.position();
      Offset offset=Board_Table.getShortOffset(Offset32(from,to));
      assert(!offset.zero());
      for(Position pos=to+offset;pos!=from;pos+=offset){
	assert(state.getPieceAt(pos).isEmpty()); 
	// ư
	typedef effect_action::SimpleMove<move_action::NotKingOpenFilter<P,Action> > simple_action_t;
	simple_action_t simpleAction(safe_action);
	state.template forEachEffectNotBy<P ,simple_action_t>(pos,p,simpleAction);
	// ֤
	generateDropAll(state,pos,action);
      }
    }

    template<Player P,class Action,bool CheapOnly>
    void Escape<P,Action,CheapOnly>::
    generateBlocking(const NumEffectState& state,Piece p,Position from,Action &action)
    {
      Position to=p.position();
      generateBlocking(state, p, to, from, action);
    }

    template<Player P,class Action,bool CheapOnly>
    void Escape<P,Action,CheapOnly>::
    blockByMoveOne(const NumEffectState& state, Position pos, Action &action)
    {
      const PieceMask pieces = state.getEffect(pos) & state.getOnBoardMask(P);
      int offset = 0;
      mask_t m = pieces.selectBit<PAWN>();
      if (m.none()) { 
	m = pieces.selectBit<LANCE>(); 
	offset = PtypeFuns<LANCE>::indexNum*32; 
      }
      if (m.none()) { 
	m = pieces.selectBit<KNIGHT>(); 
	offset = PtypeFuns<KNIGHT>::indexNum*32; 
      }
      if (m.none()) { 
	m = pieces.selectBit<SILVER>(); 
	offset = PtypeFuns<SILVER>::indexNum*32; 
      }
      if (m.none()) { 
	m = pieces.selectBit<GOLD>(); 
	offset = PtypeFuns<GOLD>::indexNum*32; 
      }
      if (m.none()) { 
	m = pieces.selectBit<BISHOP>(); 
	offset = PtypeFuns<BISHOP>::indexNum*32; 
      }
      if (m.none()) { 
	m = pieces.selectBit<ROOK>(); 
	offset = PtypeFuns<ROOK>::indexNum*32; 
      }
      if (m.none()) 
	return;
      typedef effect_action::SimpleMove<Action> simple_action_t;
      simple_action_t simpleAction(action);
      const Piece p = state.getPieceOf(m.takeOneBit() + offset);
      simpleAction.template doAction<P>(p, pos);
    }
    
    template<Player P,class Action,bool CheapOnly>
    void Escape<P,Action,CheapOnly>::
    generateBlocking(const NumEffectState& state,Piece p,Position to,Position from,Action &action)
    {
      assert(from.isOnBoard());
      Offset offset=Board_Table.getShortOffset(Offset32(from,to));
      assert(!offset.zero());
      for(Position pos=to+offset;pos!=from;pos+=offset){
	assert(state.getPieceAt(pos).isEmpty());
	if (! CheapOnly) {
	  // ư
	  typedef effect_action::SimpleMove<Action> simple_action_t;
	  simple_action_t simpleAction(action);
	  state.template forEachEffectNotBy<P ,simple_action_t>(pos,p,simpleAction);
	  // ֤
	  generateDropAll(state,pos,action);
	}
	else {
	  // ư
	  const int e = state.countEffect(P, pos);
	  if (e >= 2) 
	    blockByMoveOne(state, pos, action);
	  // ֤
	  if (e)
	    generateDropAll(state,pos,action);
	}
      }
    }

    template<Player P,class Action,bool CheapOnly>
    template<Ptype Type>
    void Escape<P,Action,CheapOnly>::
    generateMovesBy(const NumEffectState& state,Piece p,Piece const attackerPiece,Action& action)
    {
      if(attackerPiece==Piece::EMPTY()){
	/** escape only */
	generateEscape<Type>(state,p,action);
      }
      else if(Type == KING){
	Position attackFrom=attackerPiece.position();
	move_action::NotKingOpenFilter<P,Action> safe_action(state,action);

	generateCaptureKing( state, p, attackFrom, safe_action );
	/** escape */
	generateEscape<Type>( state,p,action);
	/** 礤 */
	generateBlockingKing(state,p,attackFrom,action,safe_action);
      }
      else{
	Position attackFrom=attackerPiece.position();
	generateCapture( state, p, attackFrom, action );
	/** escape */
	generateEscape<Type>( state,p,action);
	/** 礤 */
	generateBlocking(state,p,attackFrom,action);
      }
    }

    template<Player P,class Action,bool CheapOnly>
    void Escape<P,Action,CheapOnly>::
    generateKingEscape(const NumEffectState& state,Action& action){
      Piece kingPiece=state.getPieceOf(KingTraits<P>::index);
      Piece attackerPiece;
#ifndef NDEBUG
      const bool isAttacked=
#endif
	state.template findCheckPiece<P>(attackerPiece);
      assert(isAttacked); // ꤫ʤΤ˸Ƥ֤
      generateMovesBy<KING>(state,kingPiece,attackerPiece,action);
    }
    template<Player P,class Action,bool CheapOnly>
    template<Ptype Type>
    void Escape<P,Action,CheapOnly>::
    generateMovesBy(const NumEffectState& state,Piece p,Action& action)
    {
      Position target=p.position();
      Piece attackerPiece;
      bool isAttacked=
	state.template hasEffectBy<PlayerTraits<P>::opponent>(target,attackerPiece);
      assert(isAttacked); // ꤫ʤΤ˸Ƥ֤
      generateMovesBy<Type>(state,p,attackerPiece,action);
    }

    template<Player P,class Action,bool CheapOnly>
    void Escape<P,Action,CheapOnly>::
    generateMoves(const NumEffectState& state,Piece piece,Piece attackerPiece,Action& action)
    {
      switch(piece.ptype()){
      case PAWN: generateMovesBy<PAWN>(state,piece,attackerPiece,action); break;
      case LANCE: generateMovesBy<LANCE>(state,piece,attackerPiece,action); break;
      case KNIGHT: generateMovesBy<KNIGHT>(state,piece,attackerPiece,action); break;
      case SILVER: generateMovesBy<SILVER>(state,piece,attackerPiece,action); break;
      case PPAWN: generateMovesBy<PPAWN>(state,piece,attackerPiece,action); break;
      case PLANCE: generateMovesBy<PLANCE>(state,piece,attackerPiece,action); break;
      case PKNIGHT: generateMovesBy<PKNIGHT>(state,piece,attackerPiece,action); break;
      case PSILVER: generateMovesBy<PSILVER>(state,piece,attackerPiece,action); break;
      case GOLD: generateMovesBy<GOLD>(state,piece,attackerPiece,action); break;
      case BISHOP: generateMovesBy<BISHOP>(state,piece,attackerPiece,action); break;
      case PBISHOP: generateMovesBy<PBISHOP>(state,piece,attackerPiece,action); break;
      case ROOK: generateMovesBy<ROOK>(state,piece,attackerPiece,action); break;
      case PROOK: generateMovesBy<PROOK>(state,piece,attackerPiece,action); break;
      case KING: generateMovesBy<KING>(state,piece,attackerPiece,action); break;
      default: assert(0);
	
      }
    }
    template<Player P,class Action,bool CheapOnly>
    void Escape<P,Action,CheapOnly>::
    generateMoves(const NumEffectState& state,Piece piece,Action& action)
    {
      assert(piece.owner() == P);
      Position target=piece.position();
      Piece attackerPiece;
      state.template hasEffectBy<PlayerTraits<P>::opponent>(target,attackerPiece);
      generateMoves(state,piece,attackerPiece,action);
    }
  } // namespace move_generator

} // namespace osl

#endif // _GENERATE_ESCAPE_MOVES_TCC
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:

