#ifndef _GENERATE_PIECE_MOVES_H
#define _GENERATE_PIECE_MOVES_H
#include "osl/move_generator/promoteType.h"
#include "osl/move_action/concept.h"
#include "osl/misc/loki.h"
#include "osl/container/bitXmask.h"
#include "osl/state/numEffectState.h"
#include <boost/static_assert.hpp>

namespace osl
{
  namespace container
  {    
    class MoveVector;
  }
  namespace move_generator
  {
    namespace bits
    {
      /**
       * PieceOnBoard ǰĹư򰷤
       */
      template <Player P,Ptype Type,PromoteType CanP,Direction Dir>
      struct PieceOnBoardLong
      {
	BOOST_STATIC_ASSERT(DirectionTraits<Dir>::isLong);
	template <class Action>
	static void generate(const NumEffectState& state,
			     Position pos, Action& action);
      };
    
      /**
       * PieceOnBoard ΰûư򰷤
       */
      template <Player P,Ptype T,PromoteType CanP,Direction Dir>
      struct PieceOnBoardShort
      {
	BOOST_STATIC_ASSERT(! DirectionTraits<Dir>::isLong);
	template <class Action>
	static void generate(const NumEffectState& state,
			     Position pos, Action& action)
	{
	  // BOOST_FUNCTION_REQUIRE(Action,osl::move_action,Concept);
	  const Offset offset=DirectionPlayerTraits<Dir,P>::offset();
	  Position toPos=pos+offset;
	  Piece p1=state.getPieceAt(toPos);
	  if (p1.canMoveOn<P>())
	  {
	    if (CanP==CanPromoteType 
		|| (CanP==CheckPromoteType
		    && DirectionTraits<Dir>::canPromoteTo 
		    && toPos.canPromote<P>()))
	    {
	      action.unknownMove(pos,toPos,p1,PtypeFuns<T>::promotePtype,true,P);
	      if (PtypePlayerTraits<T,P>::canDropTo(toPos))
		action.unknownMove(pos,toPos,p1,T,false,P);
	    }
	    else
	    {
	      action.unknownMove(pos,toPos,p1,T,false,P);
	    }
	  }
	}
      };

      /**
       * PieceOnBoard ΰ Dir Ѥư.
       * Dir ˹ˡŪư뤳Ȥǧ
       */
      template <Player P,Ptype T,PromoteType CanP,Direction Dir>
      struct PieceOnBoardDirUnsafe
      {
	// BOOST_STATIC_ASSERT((PtypeTraits<MoveT>::moveMask & DirectionTraits<Dir>::mask));
	template <class Action>
	static void generate(const NumEffectState& state,
			     Position pos, Action& action,Int2Type<true>)
	{
	  PieceOnBoardLong<P,T,CanP,Dir>::generate(state,pos,action);
	}

	template <class Action>
	static void generate(const NumEffectState& state,
			     Position pos, Action& action,Int2Type<false>)
	{
	  PieceOnBoardShort<P,T,CanP,Dir>::generate(state,pos,action);
	}

	template <class Action>
	static void generate(const NumEffectState& state,
			     Position pos, Action& action)
	{
	  generate(state,pos,action,
		   Int2Type<DirectionTraits<Dir>::isLong>());
	}
      };
    } // bits

    template <Player P,Ptype T,Ptype MoveT,PromoteType CanP,Direction Dir>
    struct PieceOnBoardDir
    {
    private:
      template <class Action>
      static void generate(const NumEffectState& state,
			   Position pos, Action& action,Int2Type<true>)
      {
	bits::PieceOnBoardDirUnsafe<P,T,NoPromoteType,Dir>::
	  generate(state,pos,action);
      }

      template <class Action>
      static void generate(const NumEffectState& state,Position pos, Action& action,Int2Type<false>)
      {
	bits::PieceOnBoardDirUnsafe<P,T,CanP,Dir>::
	  generate(state,pos,action);
      }

      template <class Action>
      static void generateIfValid(const NumEffectState& state,
				  Position pos, Action& action, Int2Type<true>)
      {
	generate(state,pos,action,
		 Int2Type<(CanP == CheckPromoteType 
			   && !DirectionTraits<Dir>::canPromoteTo)>());
      }
      template <class Action>
      static void generateIfValid(const NumEffectState&,
				  Position, Action&, Int2Type<false>)
      {
      }
    public:
      template <class Action>
      static void generate(const NumEffectState& state,
			   Position pos, Action& action)
      {
	generateIfValid(state,pos,action,
			Int2Type<(PtypeTraits<MoveT>::moveMask 
				  & DirectionTraits<Dir>::mask) !=0>());
      }
    };
    
    /**
     * PieceOnBoard ΰPtype ¾
     */
    template <Player P,Ptype Type,Ptype MoveType,PromoteType CanP>
    struct PieceOnBoardTypePromotion
    {
      template <class Action>
      static void generate(const NumEffectState& state,
			   Position pos, Action& action);
    };


    /**
     * PieceOnBoard ΰPtype 
     */
    template <Player P, Ptype T>
    struct PieceOnBoardType
    {
      // BOOST_CLASS_REQUIRE(Action,osl::move_action,Concept);
      template <class Action>
      static void generate(const NumEffectState& state,Piece p, Action& action,
			   Int2Type<true>)
      {
	const Position pos=p.position();
	if (p.isPromotedNotKingGold())
	{
	  PieceOnBoardTypePromotion<P,PtypeFuns<T>::promotePtype,PtypeTraits<PtypeFuns<T>::promotePtype>::moveType,NoPromoteType>::
	    generate(state,pos,action);
	}
	else
	{
	  if (pos.canPromote<P>())
	    PieceOnBoardTypePromotion<P,T,T,CanPromoteType>::
	      generate(state,pos,action);
	  else
	    PieceOnBoardTypePromotion<P,T,T,CheckPromoteType>::
	      generate(state,pos,action);
	}
      }
      template <class Action>
      static void generate(const NumEffectState& state,Piece p, Action& action,
			   Int2Type<false>)
      {
	const Position pos=p.position();
	PieceOnBoardTypePromotion<P,T,T,NoPromoteType>
	  ::generate(state,pos,action);
      }

      template <class Action>
      static void generate(const NumEffectState& state,Piece p, Action& action)
      {
	generate(state,p,action,Int2Type<PtypeTraits<T>::canPromote>());
      }
    };
    
    /**
     * pieceư
     */
    template <Player P>
    struct PieceOnBoard
    {
      //BOOST_CLASS_REQUIRE(Action,osl::move_action,Concept);
      template <class Action>
      static void generateMovesPawn(const NumEffectState& state,Piece p, Action& action,BitXmask& pawnMask)
      {
	Position pos=p.position();
	if (p.isPromotedNotKingGold())
	{
	  PieceOnBoardTypePromotion<P,PPAWN,GOLD,NoPromoteType>::
	    generate(state,pos,action);
	}
	else
	{
	  pawnMask.set(pos.x());
	  if (pos.canPromote<P>())
	    PieceOnBoardTypePromotion<P,PAWN,PAWN,CanPromoteType>::
	      generate(state,pos,action);
	  else
	    PieceOnBoardTypePromotion<P,PAWN,PAWN,CheckPromoteType>::
	      generate(state,pos,action);
	}
      }

      /**
       * Generate moves without stating the Ptype as template param.
       */
      template <class Action>
      static void generate(const NumEffectState& state,Piece p, Action& action);

    };

    struct GeneratePieceOnBoard
    {
      static void generate(Player turn, const NumEffectState& state, Piece target,
			   container::MoveVector&);
    };
  } // namespace move_generator
  using move_generator::GeneratePieceOnBoard;
} // namespace osl

#endif /* _GENERATE_PIECE_MOVES_H */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
