#ifndef _OSL_RECORD_KANJI_CODE_H
#define _OSL_RECORD_KANJI_CODE_H

#include <string>

namespace osl
{
  namespace record
  {
    /**
     * Define Kanji characters by using EUC-JP codes.
     */
    const static std::string K_BLACK_SIGN  = "\xA2\xA5"; // 
    const static std::string K_WHITE_SIGN  = "\xA2\xA4"; // 
    const static std::string K_SPACE       = "\xA1\xA1"; // ""
    const static std::string K_NAKAGURO    = "\xA1\xA6"; // 
    const static std::string K_COLON       = "\xA1\xA7"; // 
    const static std::string K_R1          = "\xA3\xB1"; // 
    const static std::string K_R2          = "\xA3\xB2"; // 
    const static std::string K_R3          = "\xA3\xB3"; // 
    const static std::string K_R4          = "\xA3\xB4"; // 
    const static std::string K_R5          = "\xA3\xB5"; // 
    const static std::string K_R6          = "\xA3\xB6"; // 
    const static std::string K_R7          = "\xA3\xB7"; // 
    const static std::string K_R8          = "\xA3\xB8"; // 
    const static std::string K_R9          = "\xA3\xB9"; // 
    const static std::string K_K1          = "\xB0\xEC"; // 
    const static std::string K_K2          = "\xC6\xF3"; // 
    const static std::string K_K3          = "\xBB\xB0"; // 
    const static std::string K_K4          = "\xBB\xCD"; // 
    const static std::string K_K5          = "\xB8\xDE"; // 
    const static std::string K_K6          = "\xCF\xBB"; // ϻ
    const static std::string K_K7          = "\xBC\xB7"; // 
    const static std::string K_K8          = "\xC8\xAC"; // Ȭ
    const static std::string K_K9          = "\xB6\xE5"; // 
    const static std::string K_MIGI        = "\xB1\xA6"; // 
    const static std::string K_HIDARI      = "\xBA\xB8"; // 
    const static std::string K_UE          = "\xBE\xE5"; // 
    const static std::string K_SHITA       = "\xB2\xBC"; // 
    const static std::string K_SUGU        = "\xC4\xBE"; // ľ
    const static std::string K_YORU        = "\xB4\xF3"; // 
    const static std::string K_HIKU        = "\xB0\xFA"; // 
    const static std::string K_YUKU        = "\xB9\xD4"; // 
    const static std::string K_ONAZI       = "\xC6\xB1"; // Ʊ
    const static std::string K_NARU        = "\xC0\xAE"; // 
    const static std::string K_FUNARI      = "\xC9\xD4" + K_NARU; // 
    const static std::string K_UTSU        = "\xC2\xC7"; // 
    // pieces
    const static std::string K_PAWN        = "\xCA\xE2"; // 
    const static std::string K_PAWN_R      = "\xA7\xAE"; // 
    const static std::string K_PPAWN       = "\xA4\xC8"; // 
    const static std::string K_PPAWN_R     = "\xA7\xAF"; // 
    const static std::string K_LANCE       = "\xB9\xE1"; // 
    const static std::string K_LANCE_R     = "\xA7\xAC"; // 
    const static std::string K_PLANCE      = "\xB0\xC9"; // 
    const static std::string K_PLANCE_R    = "\xA7\xAD"; // 
    const static std::string K_PLANCE_D    = K_NARU + K_LANCE; // 
    const static std::string K_KNIGHT      = "\xB7\xCB"; // 
    const static std::string K_KNIGHT_R    = "\xA7\xAA"; // 
    const static std::string K_PKNIGHT     = "\xB7\xBD"; // 
    const static std::string K_PKNIGHT_R   = "\xA7\xAB"; // 
    const static std::string K_PKNIGHT_D   = K_NARU + K_KNIGHT; // 
    const static std::string K_SILVER      = "\xB6\xE4"; // 
    const static std::string K_SILVER_R    = "\xA7\xA8"; // 
    const static std::string K_PSILVER     = "\xC1\xB4"; // 
    const static std::string K_PSILVER_R   = "\xA7\xA9"; // 
    const static std::string K_PSILVER_D   = K_NARU + K_SILVER; // 
    const static std::string K_GOLD        = "\xB6\xE2"; // 
    const static std::string K_GOLD_R      = "\xA7\xA7"; // 
    const static std::string K_BISHOP      = "\xB3\xD1"; // 
    const static std::string K_BISHOP_R    = "\xA7\xA4"; // 
    const static std::string K_PBISHOP     = "\xC7\xCF"; // 
    const static std::string K_PBISHOP_R   = "\xA7\xA5"; // 
    const static std::string K_ROOK        = "\xC8\xF4"; // 
    const static std::string K_ROOK_R      = "\xA7\xA2"; // 
    const static std::string K_PROOK       = "\xCE\xB6"; // ζ
    const static std::string K_PROOK2      = "\xCE\xB5"; // ε
    const static std::string K_PROOK_R     = "\xA7\xA3"; // 
    const static std::string K_KING        = "\xB6\xCC"; // 
    const static std::string K_KING_R      = "\xA7\xA1"; // 
    //
    const static std::string K_BLACK       = "\xC0\xE8\xBC\xEA"; // 
    const static std::string K_WHITE       = "\xB8\xE5\xBC\xEA"; // 
    const static std::string K_BLACK_STAND = K_BLACK + "\xBB\xFD\xB6\xF0"; // 
    const static std::string K_WHITE_STAND = K_WHITE + "\xBB\xFD\xB6\xF0"; // 
    const static std::string K_TEAIWARI    = "\xBC\xEA\xB9\xE7\xB3\xE4"; // 

  } // namespace record
} // namespace osl


#endif /* _OSL_RECORD_KANJI_CODE_H */
/* ------------------------------------------------------------------------- */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
