#ifndef _OPENING_BOOK_H
#define _OPENING_BOOK_H
#include "osl/move.h"
#include "osl/stl/vector.h"
#include "osl/state/simpleState.h"
#include "osl/record/compactBoard.h"
#include <fstream>

namespace osl
{
  namespace record
  {
    namespace opening
    {
      class OMove
      {
      public:
	OMove(int i) { value = i; }
	OMove(Move m)
	{ 
	  const Position from = m.from();
	  const Position to = m.to();
	  const int bitFrom = (from.isPieceStand() ? 0 : 
			       (from.x() << 4 | from.y()));
	  const int bitTo = (to.isPieceStand() ? 0 : 
			     (to.x() << 12 | to.y() << 8));
	  value = (bitFrom | bitTo |
		   static_cast<unsigned int>(m.isPromote()) << 19 |
		   static_cast<unsigned int>(m.capturePtype()) << 20 |
		   static_cast<unsigned int>(m.ptype()) << 24 |
		   static_cast<int>(m.player()) << 28);
	}
	Position getFrom()
	{
	  if ((value & 0xff) == 0)
	    return Position::STAND();
	  else
	    return Position((value >> 4) & 0xf, value & 0xf);
	}
	Position getTo()
	{
	  if (((value >> 8) & 0xff) == 0)
	    return Position::STAND();
	  else
	    return Position((value >> 12) & 0xf, (value >> 8) & 0xf);
	}
	bool isPromote()
	{
	  return (value >> 19) & 1;
	}
	Ptype getCapturePtype()
	{
	  return static_cast<Ptype>((value >> 20) & 0xf);
	}
	Ptype getPtype()
	{
	  return static_cast<Ptype>((value >> 24) & 0xf);
	}
	Player getPlayer()
	{
	  return static_cast<Player>((value) >> 28);
	}

	operator Move() { return Move(getFrom(), getTo(), getPtype(),
					 getCapturePtype(), isPromote(),
					 getPlayer()); }
	operator int() { return value; }
      private:
	int value;
      };

      class OBMove
      {
	Move move;
	int stateIndex;
      public:
	OBMove(Move m,int i) :move(m),stateIndex(i) {}
	Move getMove() const { return move; }
	int getStateIndex() const { return stateIndex; }
      };

      /**
       * StateOBMoveݻ.
       * Statevector<OBMove>ȹ鸫winCount, loseCountݻ
       * OBMoveMoveȤMoveѤStateindex
       * ե
       * state - 4byte
       * State - 16byte * state
       *   + winCount
       *   + winCount
       *   + OBMoveο 
       *   + OBMoveγindex
       * OBMove - 8byte * OBMove
       *   + Move (4byte)
       *   + Stateindex
       */
      class WinCountBook
      {
	int nStates;
	std::ifstream ifs;
      public:
	WinCountBook(const char *filename);
	~WinCountBook();
	int getWinCount(int stateIndex);
	int getLoseCount(int stateIndex);
	vector<OBMove> getMoves(int stateIndex);
      private:
	int readInt();
	void seek(int offset);
      };

      class WMove;
      std::ostream& operator<<(std::ostream& os, const WMove& w);
      std::istream& operator>>(std::istream& is, WMove& w);
      class WMove
      {
	Move move;
	int stateIndex;
	int weight;
      public:
	WMove() {}
	WMove(Move m, int i, int w)
	  : move(m), stateIndex(i), weight(w) {}
	Move getMove() const { return move; }
	int getStateIndex() const { return stateIndex; }
	int getWeight() const { return weight; }
        void setWeight(const int w) { weight = w; };
	friend std::ostream& operator<<(std::ostream& os, const WMove& w);
	friend std::istream& operator>>(std::istream& is, WMove& w);
      };
      inline bool operator==(const WMove& l, const WMove& r) 
      {
	return l.getMove() == r.getMove() && l.getStateIndex() == r.getStateIndex()
	  && l.getWeight() == r.getWeight();
      }

      /**
       * WMoveWeightˤsort
       */
      struct WMoveSort : public std::binary_function<WMove, WMove, bool>
      {
	bool operator()(const WMove& l, const WMove& r) const 
	{
	  return l.getWeight() > r.getWeight();
	}
      };

      /**
       * WMoveMoveˤsort
       */
      struct WMoveMoveSort : public std::binary_function<WMove, WMove, bool>
      {
        bool operator()(const WMove& l, const WMove& r) const 
        {
          return l.getMove().intValue() < r.getMove().intValue();
        }
      };

      /**
       * WMoveWeightMoveˤsort
       */
      struct WMoveWeightMoveSort : public std::binary_function<WMove, WMove, bool>
      {
        bool operator()(const WMove& l, const WMove& r) const 
        {
	  if (l.getWeight() != r.getWeight())
	    return l.getWeight() > r.getWeight();
          return l.getMove().intValue() < r.getMove().intValue();
        }
      };

      /**
       * StateWMoveݻ.
       * Statevector<WMove>ݻ
       * WMoveMoveȤMoveѤStateindexȼ֤鸫
       * MoveνŤ(0-1000)
       * ե
       * versionֹ - 4byte
       * state - 4byte
       * move - 4byte
       * state index - 4byte
       * State - 16byte * state
       *   + WMoveγindex
       *   + WMoveο 
       *   + ξ
       *   + ξ
       * WMove - 12byte * WMove
       *   + Move (4byte)
       *   + Stateindex
       *   + Weight
       * CompactBoard - 164byte * state
       */
      class WeightedBook
      {
	int nStates;
	int nMoves;
	int startState;
	std::ifstream ifs;
      public:
        typedef vector<WMove> WMoveContainer;

	WeightedBook(const char *filename);
	~WeightedBook();
        /**
         * Return moves from the state of the stateIndex. If the zero_include is
         * true, all of the moves are returned. Otherwise, the moves that
         * have some weights (i.e. non-zero value) are returned.
         */
	WMoveContainer getMoves(int stateIndex, const bool zero_include = true);
	int getWhiteWinCount(int stateIndex);
	int getBlackWinCount(int stateIndex);
        osl::record::CompactBoard getCompactBoard(int stateIndex);
	SimpleState getBoard(int stateIndex);
	int getTotalState() const { return nStates; }
	int getStartState() const { return startState; }
	void validate();
        /** 
         * As traversing the 'tree', return all state indices of the state's
         * parents. 
         * @return state indexes; empty if there is none.
         */
        std::vector<int> getParents(const int stateIndex);
        /** 
         * As traversing the 'tree', find a state index of the state.  If
         * the visit_zero is true zero-weighted moves are visited (in this
         * case, player is ignored). Otherwise, the palyer's zero-weighted
         * moves are not visited.
         *
         * @param state to find
         * @param visit_zero
         * @param player
         * @return a state index of the state; if it is not found, return -1.
         */
        int getStateIndex(const SimpleState& state, 
                          const bool visit_zero = true, 
                          const Player player = BLACK);
        /** 
         * As traversing the 'tree', find a state index of the state reached
         * by applying the moves from the initial state.
         * Note that zero-weighted moves are visited.
         *
         * @param moves to apply
         * @return state index; if it is not found, return -1.
         */
        int getStateIndex(const vector<Move>& moves);
      private:
	void seek(int offset);
	static const int HEADER_SIZE = 16;
	static const int STATE_SIZE = 16;
	static const int MOVE_SIZE = 12;
	static const int BOARD_SIZE = 41 * 4;
      };
    }
  } // namespace record
} // namespace osl
#endif // _OPENING_BOOK_H
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
