/* alphaBeta2.h
 */
#ifndef _ALPHA_BETA2_H
#define _ALPHA_BETA2_H

#include "osl/search/realizationProbability.h"
#include "osl/search/searchBase.h"
#include "osl/search/searchState2.h"
#include "osl/search/searchRecorder.h"
#include "osl/search/passCounter.h"
#include "osl/search/killerMoveTable.h"
#include "osl/search/searchTimer.h"
#include "osl/checkmate/dualCheckmateSearcher.h"
#include "osl/eval/evalTraits.h"
#include "osl/eval/ml/testEval.h"
#include "osl/eval/progressEval.h"
#include "osl/container/moveStack.h"
#include "osl/container/moveLogProbVector.h"
#include "osl/stat/average.h"
#include "osl/oslConfig.h"
#include <boost/scoped_array.hpp>
#include <boost/noncopyable.hpp>
#include <iosfwd>

namespace osl
{
  namespace search
  {
    class SimpleHashRecord;
    class SimpleHashTable;
    class MoveGenerator;
    class MoveWithComment;

    class AlphaBeta2Window
    {
      CArray<int,2> values;
    public:
      explicit AlphaBeta2Window(int a=0) { values.fill(a); }
      AlphaBeta2Window(int a, int b) 
      {
	values[0] = a;
	values[1] = b;
      }
      AlphaBeta2Window(Player P, int a=0, int b=0) 
      {
	alpha(P) = a;
	beta(P) = b;
      }
      int& alpha(Player P) { return values[P]; }
      int& beta(Player P) { return values[alt(P)]; }

      int alpha(Player P) const { return values[P]; }
      int beta(Player P) const { return values[alt(P)]; }
      bool isConsistent() const { 
	return eval::notLessThan(BLACK, beta(BLACK), alpha(BLACK));
      }
      bool null() const { return values[0] == values[1]; }
    };

    /**
     * AlphaBeta2Tree ΥǡСdefault copy constructor copyǽʤΡ
     */
    template <class EvalT>
    struct AlphaBeta2Common
#if OSL_WORDSIZE == 32
      : public misc::Align16New
#endif
    {
      static const int leaf_limit = 300;

      enum { MaxDepth = SearchState2Core::MaxDepth };
      EvalT eval;
      PassCounter pass_count;
      enum MoveType { INITIAL, HASH=INITIAL, TACTICAL, KILLER, PASS, ALL, FINISH };
      /** ߤοǤκ, nextMove()  */
      CArray<MoveType, MaxDepth> move_type;
      CArray<bool, MaxDepth> in_pv;
      typedef FixedCapacityVector<Move,4> killer_t;
      CArray<killer_t, MaxDepth> killers;
      const MoveVector *root_ignore_moves; // acquaintance
      
      explicit AlphaBeta2Common(const NumEffectState& s) 
	: eval(s), root_ignore_moves(0)
      {
      }
    };
    struct AlphaBeta2SharedRoot
    {
      vector<int> last_root_value;
      Move last_root_move;
      int last_root_value_update;
      SearchState2::PVVector last_pv;
      AlphaBeta2SharedRoot() : last_root_value_update(0)
      {
      }
    };

    template <class EvalT> class AlphaBeta2Parallel;
    /**
     * "tree" of AlphaBeta2, copied by split
     */
    template <class EvalT>
    class AlphaBeta2Tree
      : public SearchBase<EvalT,SimpleHashTable,CountRecorder,RealizationProbability>,
	public SearchState2, public SearchTimer, protected AlphaBeta2Common<EvalT>, boost::noncopyable
    {
    public:
      typedef EvalT eval_t;
      typedef AlphaBeta2Common<EvalT> common_t;
      enum { MaxDepth = SearchState2Core::MaxDepth };
    protected:
      /** Żõޤ᤿Ρɿ */
      size_t node_count;
      FixedCapacityVector<MoveGenerator*, MaxDepth> generators;
      stat::Average mpn, mpn_cut, alpha_update, last_alpha_update;
      stat::Average ext, ext_limit;
      boost::shared_ptr<AlphaBeta2Parallel<EvalT> > shared;
      boost::shared_ptr<AlphaBeta2SharedRoot> shared_root;
    protected:
      static CArray<int, SearchState2Core::MaxDepth> depth_node_count;
      AlphaBeta2Tree(const NumEffectState& s, checkmate_t& checker,
		     SimpleHashTable *t, CountRecorder&);
      // share parallel data for split
      AlphaBeta2Tree(const AlphaBeta2Tree& src, AlphaBeta2Parallel<EvalT> *);
      ~AlphaBeta2Tree();
    private:
      void throwStop();
    public:
      struct BetaCut {};
      bool stopping() const
      {	
	return stop_tree || SearchTimer::stopping();
      }
      void testStop() { 
	throwIfNoMoreTime(this->recorder.allNodeCount());
	if (stop_tree)
	  throw BetaCut();
      }
    public:
      typedef AlphaBeta2Window Window;
      size_t nodeCount() const { return node_count; }
      static int rootAlpha(Player P, int last_value);
      static int stableThreshold(Player P, int last_value);

      template <Player P>
      const MoveLogProb nextMove();
    protected:
      void updateRootPV(Player P, std::ostream&, int, Move);
      bool isStable(Player P, int new_value) const;
      void showFailLow(int result, Move m) const;
    private:
      void showPV(std::ostream&, int, Move, bool stable) const;
    public:
      template <Player P> struct NextMove;
      friend class NextMove<BLACK>;
      friend class NextMove<WHITE>;
      template <Player P> class NextQMove;
      friend class NextQMove<BLACK>;
      friend class NextQMove<WHITE>;
    protected:
      /**
       * alphaBetaSearch (move)
       * - makeMove(move)
       * - => alphaBetaSearchAfterMove
       * -- search extension etc.
       * -- => searchAllMoves => alphaBetaSearch (child move)
       * - unmakeMove(move)
       */
      template <Player P>
      int alphaBetaSearch(const MoveLogProb& move, Window window,
			  bool in_pv);
      template <Player P>
      int alphaBetaSearchAfterMove(const MoveLogProb& move, 
				   Window window, bool in_pv);
      template <Player P> int quiesce(Window);
      template <Player P> int quiesceStable(Window);
      template <Player P> int quiesceExp(Window);

      template <Player P>
      int searchAllMoves(SimpleHashRecord*, Window w);
      template <Player P>
      int searchAllMoves(Move m, int limit_consumption, 
			 SimpleHashRecord*, Window w);

      /** ǵͤߤɤ */
      template <Player P>
      bool tryCheckmate(SimpleHashRecord *record, bool in_pv, Move& checkmate_move);
      /** 餱ʻˤ˵ͤߤɤ */
      template <Player P>
      bool tryCheckmateAgain(SimpleHashRecord *record, Move& checkmate_move,
			     int node_count,
			     int best_value);

      /** ̵ͤͭǧ */
      template <Player P>
      void testThreatmate(SimpleHashRecord *record, bool in_pv);

      /** alphaͤޤä¾μĴ٤ */
      template <Player P>
      void examineMovesRoot(const MoveLogProbVector&, size_t, Window,
			    MoveLogProb&, int&);

      template <Player P> int quiesceRoot(Window, int depth_left, Move& best_move, DualThreatmateState);
      template <Player P> int quiesce(Window, int depth_left, DualThreatmateState);
      template <Player P>
      bool quiesceWithMove(Move, Window&, int, Move&, int&, const DualThreatmateState&);

      void updateCheckmateCount();
      bool tryPass(SimpleHashRecord *record, Player P) const;
      MoveGenerator& makeGenerator();
      static MoveGenerator *alloc();
      static void dealloc(MoveGenerator *);
#ifdef OSL_SMP
    public:
      friend class AlphaBeta2Parallel<EvalT>;
      struct NodeProperty;
      template <Player P> struct SearchJob;
      struct SearchJobData;
      struct Shared;
      friend class Shared;
      friend class SearchJob<BLACK>;
      friend class SearchJob<WHITE>;
    protected:
      template <Player P>
      void examineMovesRootPar(const MoveLogProbVector&, size_t, Window,
			       MoveLogProb&, int&);
      void examineMovesRootPar(int tree_id);
      template <Player P>
      void testMoveRoot(int tree_id, const MoveLogProb&);

      template <Player P>
      bool examineMovesOther(Window& w, MoveLogProb& best_move, int& best_value, 
			     int& tried_moves, int& alpha_update, int& last_alpha_update);
      void examineMovesOther(int tree_id);
      template <Player P>
      bool testMoveOther(int tree_id, const MoveLogProb&, size_t index, bool in_pv);
#endif
    };

    /**
     * AlphaBeta νľ
     */
    template <class EvalT>
    class AlphaBeta2
      : public AlphaBeta2Tree<EvalT>
    {
    public:
      typedef AlphaBeta2Tree<EvalT> base_t;
      typedef typename base_t::checkmate_t checkmate_t;
      typedef typename base_t::Window Window;
    public:
      AlphaBeta2(const NumEffectState& s, checkmate_t& checker,
		 SimpleHashTable *t, CountRecorder&);
      ~AlphaBeta2();

      /** 
       * entrance of alpha beta window search.
       *
       * see http://www.logos.t.u-tokyo.ac.jp/~gekisashi/algorithm/re_search.html.
       * rootDepth, curLimit are initialized here.
       */
      template <Player P>
      int alphaBetaSearchRoot(Window window, MoveLogProb& best_move,
			      int limit);
      static const Window fullWindow(Player P)
      {
	return Window(P, base_t::winThreshold(alt(P)), base_t::winThreshold(P));
      }
      int alphaBetaSearchRoot(Window window, MoveLogProb& best_move, int limit)
      {
	const Player P = this->state().getTurn();
	if (P == BLACK)
	  return alphaBetaSearchRoot<BLACK>(window, best_move, limit);
	else
	  return alphaBetaSearchRoot<WHITE>(window, best_move, limit);
      }
      int alphaBetaSearchRoot(MoveLogProb& best_move, int limit)
      {
	Window full_window = this->fullWindow(this->state().getTurn());
	return alphaBetaSearchRoot(full_window, best_move, limit);
      }
    
      /**
       * entrance of alpha beta iterative search.
       */
      Move computeBestMoveIteratively(int limit,
				      const int step=100, 
				      int initialLimit=600,
				      size_t nodeLimit=1600000,
				      const TimeAssigned& assign=TimeAssigned(MilliSeconds::Interval(60*1000)),
				      MoveWithComment *additional_info=0);
      bool isReasonableMove(Move move, int pawn_sacrifice=1);
      void setRootIgnoreMoves(const MoveVector *rim) { this->root_ignore_moves = rim; }

      static void showNodeDepth(std::ostream&);
      static void clearNodeDepth();
    public:
      void setRoot(int limit);
      void makeMove(Move);
    }; // class AlphaBeta2

  } // namespace search
  typedef search::AlphaBeta2<eval::ProgressEval> AlphaBeta2ProgressEval;
  typedef search::AlphaBeta2<eval::ml::TestEval> AlphaBeta2TestEval;
} // namespace osl

#endif /* _ALPHA_BETA2_H */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
