/* quiescenceRecord.h
 */
#ifndef _QUIESCENCERECORD_H
#define _QUIESCENCERECORD_H

#include "osl/search/dualThreatmateState.h"
#include "osl/checkmate/oracleAges.h"
#include "osl/effect_util/sendOffPosition.h"
#include "osl/ptype.h"
#include "osl/move.h"
#include "osl/container/moveVector.h"
#include "osl/stl/vector.h"
#include <iosfwd>
#ifdef OSL_SMP
#  include "osl/misc/lightMutex.h"
#endif

namespace osl
{
  namespace search
  {
    struct QSearchTraits
    {
      enum { 
	/** ̾õκ */
	MaxDepth = 8,
	/** ¨ͤο */
	CheckmateSpecialDepth = 127,
	/** »롼פο */
	HistorySpecialDepth = 126,
      };
      enum { FirstThreat = 6, SecondThreat = 2 };
      enum MoveType { 
	UNKNOWN, KING_ESCAPE, CAPTURE, PROMOTE, CHECK,
	ESCAPE, ATTACK, OTHER, 
      };
    };

    /**
     * QuiescenceSearch ǥѥͭϤʻؼ
     */
    struct QuiescenceThreat
    {
      int value;
      Move move;
      explicit QuiescenceThreat(int v=0, Move m=Move::INVALID()) 
	: value(v), move(m)
      {
      }
    };    

    struct BestMoves : public FixedCapacityVector<Move,4>
    {
      void add(Move m)
      {
	if (! m.isNormal())
	  return;
	iterator p = std::find(begin(), end(), m);
	if (p != end()) {
	  std::rotate(begin(), p, p+1);
	  return;
	}
	if (size() == capacity())
	  pop_back();
	push_back(m);
	std::rotate(begin(), begin()+(int)size()-1, begin()+(int)size());
      }
      void addSecondary(const MoveVector& v) 
      {
	FixedCapacityVector<Move,4> copy = *this;
	clear();
	if (! copy.empty())
	  push_back(copy[0]);
	for (size_t i=0; i<v.size() && size()<capacity(); ++i) {
	  assert(v[i].isNormal());
	  if (std::find(begin(), end(), v[i]) == end())
	    push_back(v[i]);
	}
	for (size_t i=1; i<copy.size() && size()<capacity(); ++i)
	  if (std::find(begin(), end(), copy[i]) == end())
	    push_back(copy[i]);
      }
    };
    
    /**
     * QuiescenceRecord ΥǡΤlock, public ʥǡʳ
     */
    struct QuiescenceRecordBase
    {
      int upper_bound, lower_bound;
      BestMoves best_moves;
      int static_value;
      /** static_value ؤζ */
      QuiescenceThreat threat1, threat2;
      int checkmate_nodes;
      int threatmate_nodes;
      mutable Position8 sendoffs;
      /** upper_bound, lower_bound, static_value õ */
      char upper_depth, lower_depth, static_value_depth;
      char static_value_type;
      mutable bool sendoffs_initialized;
    public:
      /**
       * Ȥʤ.
       * CAVEAT: -1 ȲĹ depth -1ˤʤäˤϤޤ
       */
      enum { InitialDepth = -128, };
      enum StaticValueType { UNKNOWN, UPPER_BOUND, EXACT };
    protected:
      QuiescenceRecordBase() 
	: checkmate_nodes(0), threatmate_nodes(0),
	  upper_depth(InitialDepth), lower_depth(InitialDepth),
	  static_value_depth(InitialDepth), 
	  static_value_type(UNKNOWN), sendoffs_initialized(false)
      {
      }
      ~QuiescenceRecordBase() {}
    };
    
    /**
     * QuiescenceSearch ɽ¸ǡ
     */
    class QuiescenceRecord : public QuiescenceRecordBase
    {
    public:
      static const char *toString(StaticValueType);
    private:
#ifdef OSL_SMP
      typedef osl::misc::LightMutex Mutex;
      mutable Mutex mutex;
#endif
    public:
      DualThreatmateState threatmate;
      /**  oracle  */
      AttackOracleAges attack_oracle, threatmate_oracle;
      QuiescenceRecord()
      {
      }
      /**
       * copy constructor: copy everything except for mutex
       */
      QuiescenceRecord(const QuiescenceRecord& src)
	: QuiescenceRecordBase(src), threatmate(src.threatmate),
	  attack_oracle(src.attack_oracle),
	  threatmate_oracle(src.threatmate_oracle)
      {
      }
      QuiescenceRecord& operator=(const QuiescenceRecord& src)
      {
	if (this == &src)
	  return *this;
	
	QuiescenceRecordBase::operator=(src);
	threatmate = src.threatmate;
	attack_oracle = src.attack_oracle;
	threatmate_oracle = src.threatmate_oracle;
	return *this;
      }

      template <Player Turn>
      const Position8& sendOffPosition(const NumEffectState& state) const
      {
#ifdef OSL_SMP
	Mutex::scoped_lock lk(mutex);
#endif
	assert(Turn == state.getTurn());
	if (! sendoffs_initialized)
	{
	  const Position king_position = state.getKingPosition(alt(Turn));
	  effect_util::SendOffPosition::find<Turn>(state, king_position, 
						   sendoffs);
	  sendoffs_initialized = true;
	}
	return sendoffs;
      }
      const Position8& 
      sendOffPosition(Player turn, const NumEffectState& state) const
      {
	if (turn == BLACK)
	  return sendOffPosition<BLACK>(state);
	else
	  return sendOffPosition<WHITE>(state);
      }
      /**
       * @param max recordǻȤäɤΡɿ
       * @return ;˻ȤΡɿ֤
       */
      int checkmateNodesLeft(int max)
      {
#ifdef OSL_SMP
	Mutex::scoped_lock lk(mutex);
#endif
	if (max > checkmate_nodes)
	{
	  const int left = max - checkmate_nodes;
	  checkmate_nodes = max;
	  return left;
	}
	return 0;
      }
      /**
       * @param max recordǻȤäɤΡɿ
       * @return ͤǧε;˻ȤΡɿ֤
       */
      int threatmateNodesLeft(int max)
      {
#ifdef OSL_SMP
	Mutex::scoped_lock lk(mutex);
#endif
	if (max > threatmate_nodes)
	{
	  const int left = max - threatmate_nodes;
	  threatmate_nodes = max;
	  return left;
	}
	return 0;
      }
      /** ޤǤ˵;õΡɿ */
      int checkmateNodes() const { return checkmate_nodes; }
      int threatmateNodes() const { return threatmate_nodes; }

      void clear()
      {
#ifdef OSL_SMP
	Mutex::scoped_lock lk(mutex);
#endif
	best_moves.clear();
	upper_depth = lower_depth = static_value_depth = InitialDepth;
      }
      void setStaticValue(StaticValueType type, int value, int depth,
			  const QuiescenceThreat& t1=QuiescenceThreat(),
			  const QuiescenceThreat& t2=QuiescenceThreat())
      {
#ifdef OSL_SMP
	Mutex::scoped_lock lk(mutex);
#endif
	assert((depth <= QSearchTraits::MaxDepth)
	       || (depth == QSearchTraits::CheckmateSpecialDepth));
	assert(value % 2 == 0);
	static_value = value;
	threat1 = t1;
	threat2 = t2;
	static_value_type = type;
	static_value_depth = depth;
      }
    public:
      void setLowerBound(int depth, int bound, Move best_move)
      {
#ifdef OSL_SMP
	Mutex::scoped_lock lk(mutex);
#endif
	assert((depth <= QSearchTraits::MaxDepth)
	       || (depth == QSearchTraits::CheckmateSpecialDepth));
	if (depth >= lower_depth)
	{
	  best_moves.add(best_move);
	  lower_bound = bound;
	  lower_depth = depth;
	}
      }
      void setUpperBound(int depth, int bound)
      {
#ifdef OSL_SMP
	Mutex::scoped_lock lk(mutex);
#endif
	assert((depth <= QSearchTraits::MaxDepth)
	       || (depth == QSearchTraits::CheckmateSpecialDepth));
	if (depth >= upper_depth)
	{
	  upper_bound = bound;
	  upper_depth = depth;
	}
      }
      void setHistoryValue(int value)
      {
	lower_bound = upper_bound = value;
	lower_depth = upper_depth = QSearchTraits::HistorySpecialDepth;
      }
      void setHistoryValue(Move best_move, int value)
      {
#ifdef OSL_SMP
	Mutex::scoped_lock lk(mutex);
#endif
	best_moves.add(best_move);
	setHistoryValue(value);
      }
    public:
      void addKillerMoves(const MoveVector& new_moves)
      {
#ifdef OSL_SMP
	Mutex::scoped_lock lk(mutex);
#endif	
	best_moves.addSecondary(new_moves);
      }
      StaticValueType staticValueType() const { 
	return static_cast<StaticValueType>(static_value_type); 
      }
      bool hasStaticValue() const { return staticValueType() != UNKNOWN; }
      int staticValue() const { assert(hasStaticValue()); return static_value; }
      int staticValueDepth() const { return static_value_depth; }
      int upperDepth() const { return upper_depth; }
      int lowerDepth() const { return lower_depth; }
      int upperBound() const { return upper_bound; }
      int lowerBound() const { return lower_bound; }
      const Move bestMove() const { return best_moves.empty() ? Move() : best_moves[0]; }
      int moves_size() const { 
	return std::max(0, (int)best_moves.size()-1); 
      }
      void loadMoves(MoveVector& dst) const{
#ifdef OSL_SMP
	Mutex::scoped_lock lk(mutex);
#endif
	dst.clear();
	for (size_t i=1; i<best_moves.size(); ++i)
	  dst.push_back(best_moves[i]);
      }
      void dump(std::ostream&) const;
    };
    
  } // namespace search
} // namespace osl

#endif /* _QUIESCENCERECORD_H */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
