/* simpleHashRecord.h
 */
#ifndef _SEARCH_SIMPLEHASHRECORD_H
#define _SEARCH_SIMPLEHASHRECORD_H

#include "osl/search/quiescenceRecord.h"
#include "osl/search/searchTable.h"
#include "osl/checkmate/oracleAges.h"
#include "osl/eval/evalTraits.h"
#include "osl/moveLogProb.h"
#ifdef OSL_SMP
#  include "osl/misc/lightMutex.h"
#endif
#include <cassert>

namespace osl
{
  namespace search
  {
    /**
     * SimpleHashRecord ΥǡΤlock, public ʥǡʳ
     */
    struct SimpleHashRecordBase
    {
      signed short upper_limit;
      signed short lower_limit;
      int upper_bound;
      int lower_bound;
      MoveLogProb best_move;
      /** CAVEAT: 32bit Ǥ 100/ä1֤äȹͤȰ */
      unsigned int search_nodes;
      /** moves 򹹿Ȥlimit */
      SimpleHashRecordBase()
	: upper_limit(-1), lower_limit(-1), search_nodes(0)
      {
      }
    };
    /**
     * SimpleHashTable ˵Ͽ륨ȥ
     */
    class SimpleHashRecord : private SimpleHashRecordBase
    {
#ifdef OSL_SMP
      typedef osl::misc::LightMutex Mutex;
      mutable Mutex mutex;
#endif
    public:
      /** 礤ǻȤǡ */
      QuiescenceRecord qrecord;
      bool is_king_in_check;

      SimpleHashRecord() : is_king_in_check(false)
      {
      }
      /**
       * copy constructor: copy everything except for mutex
       */
      SimpleHashRecord(const SimpleHashRecord& src)
	: SimpleHashRecordBase(src), qrecord(src.qrecord), 
	  is_king_in_check(src.is_king_in_check)
      {
      }
      SimpleHashRecord& operator=(const SimpleHashRecord& src)
      {
	if (this == &src)
	  return *this;
	
	SimpleHashRecordBase::operator=(src);
	qrecord = src.qrecord;
	is_king_in_check = src.is_king_in_check;
	return *this;
      }
      void addNodeCount(size_t diff)
      {
#ifdef OSL_SMP
	Mutex::scoped_lock lk(mutex);
#endif
	search_nodes += diff;
      }

      /**
       * ͤ񤭹. ;кꡥ
       */
      void setAbsoluteValue(Move best_move, int value, int limit)
      {
#ifdef OSL_SMP
	Mutex::scoped_lock lk(mutex);
#endif
	lower_limit = limit;
	lower_bound = value;
	upper_limit = limit;
	upper_bound = value;
	if (best_move.isNormal())
	  this->best_move = MoveLogProb(best_move, 100);
	else
	  this->best_move = MoveLogProb();
      }
      void setBestMove(Move m) 
      {
	assert(! m.isInvalid());
	// ̤ιsetLowerBoundǡ
	best_move = MoveLogProb(m, 100);
	qrecord.best_moves.add(m);
      }
      void setAbsoluteValue(MoveLogProb best_move, int value, int limit)
      {
#ifdef OSL_SMP
	Mutex::scoped_lock lk(mutex);
#endif
	lower_limit = limit;
	lower_bound = value;
	upper_limit = limit;
	upper_bound = value;
	this->best_move = best_move;
      }

      /**
       * lowerBound .
       * - ͥ
       * - Ʊʤɤͥ
       */
      void setLowerBound(Player P, int limit, const MoveLogProb& best_move, 
			 int value)
      {
#ifdef OSL_SMP
	Mutex::scoped_lock lk(mutex);
#endif
	assert((value % 2) == 0);
	assert(limit >= 0);
	if (limit < lower_limit)
	  return;
	if (best_move.validMove()
	    && ((! this->best_move.validMove())
		|| (limit > lower_limit)
		|| eval::betterThan(P, value, lower_bound)))
	{
	  this->best_move= best_move;
	}
	lower_bound = value;
	lower_limit = limit;
	if (hasUpperBound(0))
	  makeConsistent<true>(P);
      }

      /**
       * upperBound .
       * - ͥ
       * - Ʊʤ鰭ͥ
       */
      void setUpperBound(Player P, int limit, const MoveLogProb& best_move, 
			 int value)
      {
#ifdef OSL_SMP
	Mutex::scoped_lock lk(mutex);
#endif
	assert((value % 2) == 0);
	assert(limit >= 0);
	if (limit < upper_limit)
	  return;
	if (best_move.validMove()
	    && (! this->best_move.validMove()))
	{
	  // best move upper boundǤϴŪˤϹʤ
	  this->best_move= best_move;
	}
	upper_bound = value;
	upper_limit = limit;
	if (hasLowerBound(0))
	  makeConsistent<false>(P);
      }
    private:
      /**
       * ¤Ȳ¤̷⤬äĴ.
       * Ʊä夫褿ͥ褷ʤȡơõη̤ϿǤʤ
       * @param PreferLowerBound Ʊä lower bound ͥ褹
       */
      template <bool PreferLowerBound>
      void makeConsistent(Player P)
      {
	assert(hasLowerBound(0) && hasUpperBound(0));
	if (! eval::betterThan(P, lower_bound, upper_bound))
	  return;
	if ((upper_limit < lower_limit)
	    || (PreferLowerBound && (upper_limit == lower_limit)))
	{
	  upper_bound = lower_bound;
	  assert((upper_bound % 2) == 0);
	}
	else
	{
	  lower_bound = upper_bound;
	  assert((lower_bound % 2) == 0);
	}
      }
    public:
      /** upperBound ϿȤlimit */
      int upperLimit() const { return upper_limit; }
      /** ֤Υץ쥤䤫ߤɾͤξ */
      int upperBound() const { return upper_bound; }
      /** lowerBound ϿȤlimit */
      int lowerLimit() const { return lower_limit; }
      /** ֤Υץ쥤䤫ߤɾͤβ */
      int lowerBound() const { return lower_bound; }
      /** ޤǤ˵;õΡɿ */
      int checkmateNodes() const { return qrecord.checkmate_nodes; }
      int threatmateNodes() const { return qrecord.threatmate_nodes; }
      const MoveLogProb bestMove() const { 
#ifdef OSL_SMP
	Mutex::scoped_lock lk(mutex);
#endif
	return best_move; 
      }

      bool hasUpperBound(int limit) const
      {
	return (upperLimit() >= limit);
      }
      bool hasLowerBound(int limit) const
      {
	return (lowerLimit() >= limit);
      }
      bool hasAbsoluteUpperBound(Player p, int limit) const
      {
	return (p == BLACK) ? hasUpperBound(limit) : hasLowerBound(limit);
      }
      bool hasAbsoluteLowerBound(Player p, int limit) const
      {
	return (p == BLACK) ? hasLowerBound(limit) : hasUpperBound(limit);
      }
      int absoluteUpperBound(Player p) const
      {
	return (p == BLACK) ? upperBound() : lowerBound();
      }
      int absoluteLowerBound(Player p) const
      {
	return (p == BLACK) ? lowerBound() : upperBound();
      }
      void resetValue()
      {
#ifdef OSL_SMP
	Mutex::scoped_lock lk(mutex);
#endif
	lower_limit = -1;
	upper_limit = -1;
      }
      /** high fail 뤫 */
      template <Player P>
      bool hasGreaterLowerBound(int limit, int threshold, int& val) const
      {
#ifdef OSL_SMP
	Mutex::scoped_lock lk(mutex);
#endif
	if ((lowerLimit() < limit)
	    || (EvalTraits<P>::betterThan(threshold, lowerBound())))
	  return false;
	val = lowerBound();
	return true;
      }
      /** low fail 뤫 */
      template <Player P>
      bool hasLesserUpperBound(int limit, int threshold, int& val) const
      {
#ifdef OSL_SMP
	Mutex::scoped_lock lk(mutex);
#endif
	if ((upperLimit() < limit)
	    || (EvalTraits<P>::betterThan(upperBound(), threshold)))
	  return false;
	val = upperBound();
	return true;
      }

      const DualThreatmateState& threatmate() const { return qrecord.threatmate; }
      DualThreatmateState& threatmate() { return qrecord.threatmate; }

      void dump(std::ostream&) const;

      size_t nodeCount() const { return search_nodes; }
      bool inCheck() const
      {
#ifdef OSL_USE_RACE_DETECTOR
	Mutex::scoped_lock lk(mutex);
#endif
	return is_king_in_check;
      }
      void setInCheck(bool new_value)
      {
#ifdef OSL_USE_RACE_DETECTOR
	Mutex::scoped_lock lk(mutex);
#endif
	is_king_in_check = new_value;
      }
    };
  } // namespace search

  using search::SimpleHashRecord;
} // namespace osl


#endif /* _SEARCH_SIMPLEHASHRECORD_H */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
