/* hash_map.h
 */
#ifndef HASH_MAP_H
#define HASH_MAP_H

#include "osl/stl/hash.h"
#include "osl/stl/pool_allocator.h"
#if (__GNUC__ >= 4 && __GNUC_MINOR__ >=3)
#  include <tr1/unordered_map>
#else
#  include <ext/hash_map>
#endif

namespace osl
{
  namespace stl
  {
    template <class T>
    struct hash;
  
#if (__GNUC__ >= 4 && __GNUC_MINOR__ >=3)
    template<class Key, class Value, class HashFun=hash<Key>,
	     class EqualKey=std::equal_to<Key> >
    struct hash_map
      : public std::tr1::unordered_map<Key, Value, HashFun, EqualKey,
				  pool_allocator<std::pair<const Key, Value> > >
    {
      typedef std::tr1::unordered_map<Key, Value, HashFun, EqualKey,
				      pool_allocator<std::pair<const Key, Value> > > base_t;
      hash_map() {}
      hash_map(size_t s) : base_t(s)
      {
      }
      ~hash_map();
    };
    template<class Key, class Value, class HashFun, class EqualKey>
    hash_map<Key,Value,HashFun,EqualKey>::~hash_map()
    {
    }
#else
    template<class Key, class Value, class HashFun=osl::stl::hash<Key>,
	     class EqualKey=std::equal_to<Key> >
    struct hash_map
      : public __gnu_cxx::hash_map<Key, Value, HashFun, EqualKey,
				   pool_allocator<Value> >
    {
      typedef __gnu_cxx::hash_map<Key, Value, HashFun, EqualKey,
				  pool_allocator<Value> > base_t;
      hash_map() {}
      hash_map(size_t s) : base_t(s)
      {
      }
    };
#endif
  } // namespace stl
  using stl::hash_map;
} // namespace stl


#endif /* HASH_MAP_H */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
