#include "osl/checkmate/analyzer/proofTreeDepth.h"
#include "osl/checkmate/checkHashRecord.h"
#include "osl/checkmate/dfpn.h"
#include "osl/checkmate/dfpnRecord.h"
#include "osl/container/moveVector.h"
#include "osl/move_generator/legalMoves.h"
#include "osl/stl/hash_map.h"
#include "osl/stl/pointerHash.h"

/**
 * 򵭲ơ֥.
 * -1 õ
 */
struct osl::checkmate::analyzer::ProofTreeDepth::Table
  : public osl::hash_map<const CheckHashRecord*, int>
{
};

osl::checkmate::analyzer::
ProofTreeDepth::ProofTreeDepth()
  : table(new Table())
{
}

osl::checkmate::analyzer::
ProofTreeDepth::~ProofTreeDepth()
{
}

int osl::checkmate::analyzer::
ProofTreeDepth::depth(const CheckHashRecord *record, bool is_or_node) const
{
  if (! record)
  {
    assert(is_or_node);		// ImmediateCheckmate
    return 1;
  }
  assert(record);
  assert(record->proofDisproof().isCheckmateSuccess());
  return (is_or_node ? orNode(record) : andNode(record));
}

int osl::checkmate::analyzer::
ProofTreeDepth::orNode(const CheckHashRecord *record) const
{
  assert(record);
  assert(record->proofDisproof().isCheckmateSuccess());
  const Table::const_iterator p = table->find(record);
  if (p != table->end())
    return p->second;
  (*table)[record] = -1;

  if (record->bestMove && (! record->bestMove->record))
  {
    // ImmediateCheckmate
    assert(record->bestMove->flags.isSet(MoveFlags::ImmediateCheckmate));
    (*table)[record] = 1;
    return 1;
  }
  const CheckMoveList& l = record->finalByDominance()
    ? record->finalByDominance()->moves
    : record->moves;
  for (CheckMoveList::const_iterator p=l.begin(); p!=l.end(); ++p)
  {
    if (! p->record)
      continue;
    if (! p->record->proofDisproof().isCheckmateSuccess())
      continue;
    const int depth = andNode(p->record);
    if (depth >= 0)
    {
      (*table)[record] = depth+1;
      return (*table)[record];
    }
  }
  if (! l.empty())
    return -1;			// 롼¤

  (*table)[record] = 0;
  return 0;	// or node ǻؼ꤬ʤƵ => ƨƤʤ
}

int osl::checkmate::analyzer::
ProofTreeDepth::andNode(const CheckHashRecord *record) const
{
  assert(record);
  assert(record->proofDisproof().isCheckmateSuccess());
  const Table::const_iterator p = table->find(record);
  if (p != table->end())
    return p->second;
  (*table)[record] = -1;

  int result = 0;	// and node ǻؼ꤬ʤƵ => ƨʤ

  const CheckMoveList& l = record->finalByDominance()
    ? record->finalByDominance()->moves
    : record->moves;
  for (CheckMoveList::const_iterator p=l.begin(); p!=l.end(); ++p)
  {
    if (! p->record)
      continue;
    const int depth = orNode(p->record);
    if (depth < 0)
      return depth;		// loop found
    result = std::max(depth+1, result);
  }
  
  (*table)[record] = result;
  return result;
}

/* ------------------------------------------------------------------------- */

/**
 * 򵭲ơ֥.
 * -1 õ
 */
struct osl::checkmate::analyzer::ProofTreeDepthDfpn::Table
  : public osl::hash_map<HashKey, int>
{
  NumEffectState state;
  const DfpnTable& table;
  Table(const DfpnTable& t) : table(t)
  {
  }
};

osl::checkmate::analyzer::
ProofTreeDepthDfpn::ProofTreeDepthDfpn(const DfpnTable& dfpn_table)
  : table(new Table(dfpn_table))
{
}

osl::checkmate::analyzer::
ProofTreeDepthDfpn::~ProofTreeDepthDfpn()
{
}

int osl::checkmate::analyzer::
ProofTreeDepthDfpn::depth(const HashKey& key, const NumEffectState& state, bool is_or_node) const
{
  table->state = state;
  return (is_or_node ? orNode(key) : andNode(key));
}
int osl::checkmate::analyzer::
ProofTreeDepthDfpn::orNode(const HashKey& key) const
{
  const PieceStand white_stand = PieceStand(WHITE, table->state);
  DfpnRecord record = table->table.probe(key, white_stand);
  if (! record.proof_disproof.isCheckmateSuccess()) {
    // todo test immediate checkmate
    return 1;
  }
  const Table::const_iterator p = table->find(key);
  if (p != table->end())
    return p->second;
  (*table)[key] = -1;

  if (! record.best_move.isNormal())
  {
    // XXX // ImmediateCheckmate
    (*table)[key] = 1;
    return 1;
  }
  MoveVector moves;
  if (table->state.getTurn() == BLACK)
    Dfpn::generateCheck<BLACK>(table->state, moves);
  else
    Dfpn::generateCheck<WHITE>(table->state, moves);
  for (size_t i=0; i<moves.size(); ++i)
  {
    const HashKey new_key = key.newHashWithMove(moves[i]);
    const PieceStand next_white_stand = (table->state.getTurn() == WHITE) 
      ? white_stand.nextStand(WHITE, moves[i]) : white_stand;
    DfpnRecord new_record = table->table.probe(new_key, next_white_stand);
    if (! new_record.proof_disproof.isCheckmateSuccess())
      continue;
    const int depth = andNode(new_key);
    if (depth >= 0)
    {
      (*table)[new_key] = depth+1;
      return (*table)[new_key];
    }
  }
  if (! moves.empty())
    return -1;			// 롼¤

  (*table)[key] = 0;
  return 0;	// or node ǻؼ꤬ʤƵ => ƨƤʤ
}

int osl::checkmate::analyzer::
ProofTreeDepthDfpn::andNode(const HashKey& key) const
{
  const PieceStand white_stand = PieceStand(WHITE, table->state);
  DfpnRecord record = table->table.probe(key, white_stand);
  const Table::const_iterator p = table->find(key);
  if (p != table->end())
    return p->second;
  (*table)[key] = -1;

  int result = 0;	// and node ǻؼ꤬ʤƵ => ƨʤ
  MoveVector moves;
  if (table->state.getTurn() == BLACK)
    Dfpn::generateEscape<BLACK>(table->state, true, Position(), moves);
  else
    Dfpn::generateEscape<WHITE>(table->state, true, Position(), moves);

  for (size_t i=0; i<moves.size(); ++i)
  {
    const HashKey new_key = key.newHashWithMove(moves[i]);
    const int depth = orNode(new_key);
    if (depth < 0)
      return depth;		// loop found
    result = std::max(depth+1, result);
  }
  
  (*table)[key] = result;
  return result;
}

/* ------------------------------------------------------------------------- */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
