/* legal_moves.cc
 */
#include "osl/state/numEffectState.h"
#include "osl/record/csa.h"
#include "osl/record/csaRecord.h"
#include "osl/container/moveVector.h"
#include "osl/move_generator/legalMoves.h"
#include "osl/apply_move/applyMove.h"
#include "osl/record/csa.h"
#include <boost/program_options.hpp>
#include <iostream>
using namespace osl;

/**
 * @file ˡɽ
 */

int main(int argc, char **argv)
{
  bool csa;
  boost::program_options::options_description command_line_options;
  command_line_options.add_options()
    ("csa",
     boost::program_options::value<bool>(&csa)->default_value(false),
     "Show legal moves in CSA format")
    ("input-file", boost::program_options::value< std::vector<std::string> >(),
     "input files in kisen format")
    ("help", "Show help message");
  boost::program_options::variables_map vm;
  boost::program_options::positional_options_description p;
  p.add("input-file", -1);

  try
  {
    boost::program_options::store(
      boost::program_options::command_line_parser(
	argc, argv).options(command_line_options).positional(p).run(), vm);
    boost::program_options::notify(vm);
    if (vm.count("help"))
    {
      std::cerr << "Usage: " << argv[0] << " [options] CSA_FILE1 CSA_FILE2 ..."
		<< std::endl;
      std::cout << command_line_options << std::endl;
      return 0;
    }
  }
  catch (std::exception &e)
  {
    std::cerr << "error in parsing options" << std::endl
	      << e.what() << std::endl;
    std::cerr << "Usage: " << argv[0] << " [options] CSA_FILE1 CSA_FILE2 ..."
	      << std::endl;
    std::cerr << command_line_options << std::endl;
    return 1;
  }

  const std::vector<std::string> files =
    vm["input-file"].as< std::vector<std::string> >();

  for (size_t i = 0; i < files.size(); ++i)
  {
    const Record record = CsaFile(files[i]).getRecord();
    NumEffectState state(record.getInitialState());
    const vector<Move> moves=record.getMoves();
    for (vector<Move>::const_iterator p=moves.begin(); p!=moves.end(); ++p)
    {
      ApplyMoveOfTurn::doMove(state, *p);
    }
    MoveVector legal_moves;
    LegalMoves::generate(state, legal_moves);
    if (i > 0)
    {
      std::cout << std::endl;
    }
    if (csa)
    {
      for (size_t i = 0; i < legal_moves.size(); ++i)
      {
	std::cout << osl::record::csa::show(legal_moves[i]) << std::endl;
      }
    }
    else
    {
      std::cout << legal_moves;
    }
  }
}



/* ------------------------------------------------------------------------- */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
