/* checkmateIfCapture.t.cc
 */
#include "osl/checkmate/checkmateIfCapture.h"
#include "osl/record/csaString.h"
#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>

namespace osl
{
  namespace checkmate
  {
    class CheckmateIfCaptureTest : public CppUnit::TestFixture 
    {
      CPPUNIT_TEST_SUITE(CheckmateIfCaptureTest);
      // (ƥȤΤ)᥽åɤ󤹤롥
      CPPUNIT_TEST(testAttack);
      CPPUNIT_TEST_SUITE_END();
    public:
      void testAttack();
    };
  }
} // namespace osl

CPPUNIT_TEST_SUITE_REGISTRATION(osl::checkmate::CheckmateIfCaptureTest);

void osl::checkmate::CheckmateIfCaptureTest::testAttack()
{
  {
    NumEffectState state(CsaString(
			   "P1 *  *  *  *  *  * -OU *  * \n"
			   "P2 * -HI *  *  *  *  *  *  * \n"
			   "P3 *  *  *  *  * +TO+TO *  * \n"
			   "P4 *  *  *  *  *  *  *  *  * \n"
			   "P5 *  *  *  *  *  *  *  *  * \n"
			   "P6 *  *  *  *  *  *  *  *  * \n"
			   "P7 *  *  *  *  *  *  *  *  * \n"
			   "P8 *  *  *  *  *  *  *  *  * \n"
			   "P9 *  *  *  *  * +OU *  *  * \n"
			   "P+00GI\n"
			   "P-00AL\n"
			   "+\n").getInitialState());
    const Move m83(Position(8,3), SILVER, BLACK);
    CPPUNIT_ASSERT(CheckmateIfCapture::effectiveAttack(state, m83, 0));

    const Move m73(Position(7,3), SILVER, BLACK);
    CPPUNIT_ASSERT(! CheckmateIfCapture::effectiveAttack(state, m73, 0));

    // ?
    const Move m84(Position(8,4), SILVER, BLACK);
    CPPUNIT_ASSERT(CheckmateIfCapture::effectiveAttack(state, m84, 0));
  }
  {
    NumEffectState state(CsaString(
			   "P1 *  *  *  *  *  * -OU *  * \n"
			   "P2 * -HI *  *  *  *  *  *  * \n"
			   "P3 *  *  *  *  * +TO+TO *  * \n"
			   "P4 *  *  *  *  *  *  *  *  * \n"
			   "P5 *  *  *  *  *  *  *  *  * \n"
			   "P6 *  *  *  *  *  *  *  *  * \n"
			   "P7 *  *  *  *  *  *  *  *  * \n"
			   "P8 *  *  *  *  *  *  *  *  * \n"
			   "P9 * +OU *  *  *  *  *  *  * \n"
			   "P+00GI\n"
			   "P-00AL\n"
			   "+\n").getInitialState());
    const Move m83(Position(8,3), SILVER, BLACK);
    // Ȳ
    CPPUNIT_ASSERT(! CheckmateIfCapture::effectiveAttack(state, m83, 0));
  }
  {
    NumEffectState state(CsaString(
			   "P1 *  *  *  *  * -GI-OU-GI * \n"
			   "P2 * -HI *  *  *  *  *  *  * \n"
			   "P3 *  *  *  *  * +TO+TO *  * \n"
			   "P4 *  *  *  *  *  *  *  *  * \n"
			   "P5 *  *  *  *  *  *  *  *  * \n"
			   "P6 *  *  *  *  *  *  *  *  * \n"
			   "P7 *  *  *  *  *  *  *  *  * \n"
			   "P8 *  *  *  *  *  *  *  *  * \n"
			   "P9 *  *  *  *  * +OU *  *  * \n"
			   "P+00GI00GI\n"
			   "P-00AL\n"
			   "+\n").getInitialState());
    const Move m83(Position(8,3), SILVER, BLACK);
    CPPUNIT_ASSERT(! CheckmateIfCapture::effectiveAttack(state, m83, 0));

    // 3
    CPPUNIT_ASSERT(CheckmateIfCapture::effectiveAttack(state, m83, 2));

    const Move m73(Position(7,3), SILVER, BLACK);
    CPPUNIT_ASSERT(! CheckmateIfCapture::effectiveAttack(state, m73, 0));

    // ?
    const Move m84(Position(8,4), SILVER, BLACK);
    CPPUNIT_ASSERT(CheckmateIfCapture::effectiveAttack(state, m84, 2));
  }
  {
    NumEffectState state(CsaString(
			   "P1 *  *  *  *  *  *  * -KE-KY\n"
			   "P2 *  *  *  *  *  *  * -OU * \n"
			   "P3 *  *  *  *  * +TO-FU-FU * \n"
			   "P4 *  *  *  *  *  *  *  * -FU\n"
			   "P5 *  *  *  *  *  *  *  *  * \n"
			   "P6 *  *  *  *  *  *  *  *  * \n"
			   "P7 *  *  *  *  *  *  *  *  * \n"
			   "P8 *  *  *  *  *  *  *  *  * \n"
			   "P9 *  *  *  *  * +OU *  *  * \n"
			   "P+00KA00KI\n"
			   "P-00AL\n"
			   "+\n").getInitialState());
    const Move m31(Position(3,1), BISHOP, BLACK);
    CPPUNIT_ASSERT(CheckmateIfCapture::effectiveAttack(state, m31, 0));
  }
  {
    NumEffectState state(CsaString(
			   "P1 *  *  *  *  *  *  *  *  * \n"
			   "P2 *  *  *  *  *  *  *  *  * \n"
			   "P3 *  *  *  *  *  *  *  *  * \n"
			   "P4 *  *  *  *  *  *  *  *  * \n"
			   "P5 *  *  *  *  *  *  *  *  * \n"
			   "P6 *  *  *  *  * -OU *  *  * \n"
			   "P7 *  *  *  *  *  *  *  *  * \n"
			   "P8 *  *  *  *  * +OU *  *  * \n"
			   "P9 *  *  *  *  *  *  *  *  * \n"
			   "P-00AL\n"
			   "+\n").getInitialState());
    const Move m47(Position(4,8), Position(4,7), KING, PTYPE_EMPTY, false, BLACK);
    CPPUNIT_ASSERT(! CheckmateIfCapture::effectiveAttack(state, m47, 0));
  }
  {
    NumEffectState state(CsaString(
			   "P1 *  * +HI * -KI-OU * -KE-KY\n"
			   "P2 *  *  *  *  *  *  *  *  * \n"
			   "P3 *  *  * -FU-FU+FU-FU-FU-FU\n"
			   "P4 *  * +KA *  *  *  *  *  * \n"
			   "P5 *  *  *  * +KE *  *  *  * \n"
			   "P6 *  *  *  *  *  *  *  *  * \n"
			   "P7+FU+FU+FU *  *  *  *  *  * \n"
			   "P8+KY+GI+KI *  *  *  *  *  * \n"
			   "P9+OU+KE+KI *  *  *  *  *  * \n"
			   "P+00KI\n"
			   "P-00HI00KA00GI00GI00GI00KE00KY00KY00FU00FU00FU00FU00FU00FU00FU00FU00FU\n"
			   "+\n").getInitialState());
    const Move m52(Position(5,2), GOLD, BLACK);
    // ǤϤȤʤ̤Ǽȵ
    CPPUNIT_ASSERT(CheckmateIfCapture::effectiveAttack(state, m52, 0));
  }
}

/* ------------------------------------------------------------------------- */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
