/* fixedDepthSearcher.t.cc
 */
#include "osl/checkmate/fixedDepthSearcher.h"
#include "osl/record/csaRecord.h"
#include "osl/record/csaString.h"
#include "osl/oslConfig.h"

#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>
#include <iostream>
#include <fstream>

using namespace osl;
using namespace osl::checkmate;
extern bool isShortTest;

class CheckmateFixedDepthSearcherTest : public CppUnit::TestFixture 
{
  CPPUNIT_TEST_SUITE(CheckmateFixedDepthSearcherTest);
  // (ƥȤΤ)᥽åɤ󤹤롥
  CPPUNIT_TEST(testZero);
  CPPUNIT_TEST(testOne);
  CPPUNIT_TEST(testThree);
  CPPUNIT_TEST(testProofPieces);
  CPPUNIT_TEST(testSuicide);
  CPPUNIT_TEST(testCheckInCheck);
  CPPUNIT_TEST(testCheckFiles);
  CPPUNIT_TEST(testNoCheckFiles);
  CPPUNIT_TEST(testNoCheckmate);
  CPPUNIT_TEST_SUITE_END();
public:
  void testZero();
  void testOne();
  void testThree();
  void testProofPieces();
  void testSuicide();
  void testCheckInCheck();
  void testCheckFiles();
  void testNoCheckFiles();
  void testNoCheckmate();
};

CPPUNIT_TEST_SUITE_REGISTRATION(CheckmateFixedDepthSearcherTest);

void CheckmateFixedDepthSearcherTest::testZero()
{
  {
    NumEffectState state(CsaString(
			   "P1 *  *  *  *  *  *  * -KE-OU\n"
			   "P2 *  *  *  *  *  *  *  *  * \n"
			   "P3 *  *  *  *  *  *  * +TO-FU\n"
			   "P4 *  *  *  *  *  * +RY *  * \n"
			   "P5 *  *  *  *  *  *  *  *  * \n"
			   "P6 *  *  *  *  *  *  *  *  * \n"
			   "P7 *  *  *  *  *  *  *  *  * \n"
			   "P8 *  *  *  *  *  *  *  *  * \n"
			   "P9 *  * +OU *  *  *  *  *  * \n"
			   "P+00FU00FU\n"
			   "P-00AL\n"
			   "+\n").getInitialState());
    FixedDepthSearcher searcher(state);
    Move best_move;
    ProofDisproof pdp=searcher.hasCheckmateMove<BLACK>(0, best_move);
    CPPUNIT_ASSERT(!pdp.isCheckmateSuccess() && !pdp.isCheckmateFail());
  }
  {
    NumEffectState state(CsaString(
			   "P1 *  *  *  *  *  *  * -KE * \n"
			   "P2 *  *  *  *  *  *  * -OU * \n"
			   "P3 *  *  *  *  *  * +FU+TO-FU\n"
			   "P4 *  *  *  *  *  * +RY *  * \n"
			   "P5 *  *  *  *  *  *  *  *  * \n"
			   "P6 *  *  *  *  *  *  *  *  * \n"
			   "P7 *  *  *  *  *  *  *  *  * \n"
			   "P8 *  *  *  *  *  *  *  *  * \n"
			   "P9 *  * +OU *  *  *  *  *  * \n"
			   "P+00FU\n"
			   "P-00AL\n"
			   "-\n").getInitialState());
    FixedDepthSearcher searcher(state);
    Move lastMove=Move(Position(2,4),Position(2,3),PPAWN,
		       PTYPE_EMPTY,false,BLACK);
    ProofDisproof pdp=searcher.hasEscapeMove<BLACK>(lastMove,0);
    CPPUNIT_ASSERT(!pdp.isCheckmateSuccess() && !pdp.isCheckmateFail());
  }
  {
    NumEffectState state(CsaString(
			   "P1 *  *  *  *  *  *  * -KE-OU\n"
			   "P2 *  *  *  *  *  *  *  * +FU\n"
			   "P3 *  *  *  *  *  * +FU+TO-FU\n"
			   "P4 *  *  *  *  *  * +RY *  * \n"
			   "P5 *  *  *  *  *  *  *  *  * \n"
			   "P6 *  *  *  *  *  *  *  *  * \n"
			   "P7 *  *  *  *  *  *  *  *  * \n"
			   "P8 *  *  *  *  *  *  *  *  * \n"
			   "P9 *  * +OU *  *  *  *  *  * \n"
			   "P+00FU\n"
			   "P-00AL\n"
			   "-\n").getInitialState());
    FixedDepthSearcher searcher(state);
    Move lastMove=Move(Position(1,2),PAWN,BLACK);
    ProofDisproof pdp=searcher.hasEscapeMove<BLACK>(lastMove,0);
    CPPUNIT_ASSERT_EQUAL(ProofDisproof::PawnCheckmate(),pdp);
  }
}

void CheckmateFixedDepthSearcherTest::testOne()
{
  {
    NumEffectState state(CsaString(
			   "P1 *  *  *  *  *  *  * -KE-OU\n"
			   "P2 *  *  *  *  *  *  *  *  * \n"
			   "P3 *  *  *  *  *  *  * +TO-FU\n"
			   "P4 *  *  *  *  *  * +RY *  * \n"
			   "P5 *  *  *  *  *  *  *  *  * \n"
			   "P6 *  *  *  *  *  *  *  *  * \n"
			   "P7 *  *  *  *  *  *  *  *  * \n"
			   "P8 *  *  *  *  *  *  *  *  * \n"
			   "P9 *  * +OU *  *  *  *  *  * \n"
			   "P+00FU00FU\n"
			   "P-00AL\n"
			   "+\n").getInitialState());
    FixedDepthSearcher searcher(state);
    Move best_move;
    ProofDisproof pdp=searcher.hasCheckmateMove<BLACK>(1, best_move);
    CPPUNIT_ASSERT(!pdp.isCheckmateSuccess() && !pdp.isCheckmateFail());
  }
  {
    NumEffectState state(CsaString(
			   "P1 *  *  *  *  *  *  * -KE * \n"
			   "P2 *  *  *  *  *  *  * -OU * \n"
			   "P3 *  *  *  *  *  * +TO * -FU\n"
			   "P4 *  *  *  *  *  * +RY *  * \n"
			   "P5 *  *  *  *  *  *  *  *  * \n"
			   "P6 *  *  *  *  *  *  *  *  * \n"
			   "P7 *  *  *  *  *  *  *  *  * \n"
			   "P8 *  *  *  *  *  *  *  *  * \n"
			   "P9 *  * +OU *  *  *  *  *  * \n"
			   "P+00FU\n"
			   "P-00AL\n"
			   "-\n").getInitialState());
    FixedDepthSearcher searcher(state);
    Move lastMove=Move(Position(2,4),Position(3,3),PPAWN,
		       PTYPE_EMPTY,false,BLACK);
    ProofDisproof pdp=searcher.hasEscapeMove<BLACK>(lastMove,1);
    CPPUNIT_ASSERT(!pdp.isCheckmateSuccess() && !pdp.isCheckmateFail());
  }
  {
    NumEffectState state(CsaString(
			   "P1 *  *  *  *  *  *  *  *  * \n"
			   "P2 *  *  *  *  *  *  * -FU-FU\n"
			   "P3 *  *  *  *  *  *  * -KE-OU\n"
			   "P4 *  *  *  *  *  *  *  *  * \n"
			   "P5 *  *  *  *  *  *  *  *  * \n"
			   "P6 *  *  *  *  *  *  * +KE+KE\n"
			   "P7 *  *  *  *  *  *  *  *  * \n"
			   "P8 *  *  *  *  *  *  *  *  * \n"
			   "P9 *  * +OU *  *  *  *  *  * \n"
			   "P+00FU\n"
			   "P-00AL\n"
			   "+\n").getInitialState());
    FixedDepthSearcher searcher(state);
    Move best_move;
    ProofDisproof pdp=searcher.hasCheckmateMove<BLACK>(1, best_move);
    CPPUNIT_ASSERT_EQUAL(ProofDisproof::PawnCheckmate(),pdp);
  }
}

void CheckmateFixedDepthSearcherTest::testThree()
{
  {
    NumEffectState state(CsaString(
			   "P1-KY-KE *  *  * -OU * -KE-KY\n"
			   "P2 *  * +HI *  * -GI * -KI * \n"
			   "P3 *  * -KE+GI-FU-FU-FU-FU * \n"
			   "P4-FU * -FU+OU * +FU * -RY-FU\n"
			   "P5 * -FU *  * +KI *  *  *  * \n"
			   "P6+FU * +FU * +FU *  *  * +FU\n"
			   "P7 * +FU * +FU *  *  *  *  * \n"
			   "P8 *  * -KI *  * -UM *  *  * \n"
			   "P9+KY+KE *  *  *  *  *  * +KY\n"
			   "P+00KA00KI\n"
			   "P-00GI00GI00FU00FU00FU\n"
			   "+\n").getInitialState());
    FixedDepthSearcher searcher(state);
    Move best_move;
    ProofDisproof pdp=searcher.hasCheckmateMove<BLACK>(2, best_move);
    CPPUNIT_ASSERT(pdp.isCheckmateSuccess());
  }
  {
    NumEffectState state(CsaString(
			   "P1-KY-KE+GI *  *  *  *  * -KY\n"
			   "P2-OU-KE-GI * -KI *  *  *  * \n"
			   "P3 * -FU+GI *  *  * +TO * -FU\n"
			   "P4+FU * -FU-GI-HI *  *  *  * \n"
			   "P5 *  *  *  *  *  * -KA *  * \n"
			   "P6 *  * +FU+FU-FU *  *  *  * \n"
			   "P7+KE+FU+KA+KI *  *  *  * +FU\n"
			   "P8 *  * +OU *  * +KI *  *  * \n"
			   "P9+KY *  *  * +FU *  * +KE+KY\n"
			   "P+00HI00FU00FU00FU00FU00FU\n"
			   "P-00KI00FU00FU\n"
			   "+\n").getInitialState());
    FixedDepthSearcher searcher(state);
    Move best_move;
    ProofDisproof pdp=searcher.hasCheckmateMove<BLACK>(3, best_move);
    CPPUNIT_ASSERT_EQUAL(ProofDisproof::Checkmate(), pdp);
    CPPUNIT_ASSERT(best_move == Move(Position(9,4),Position(9,3),PPAWN,PTYPE_EMPTY,true,BLACK) ||
		   best_move == Move(Position(7,1),Position(8,2),PSILVER,KNIGHT,true,BLACK));
  }
  {
    NumEffectState state(CsaString(
			   "P1 *  *  *  *  *  *  *  *  * \n"
			   "P2 *  *  *  *  *  * -OU *  * \n"
			   "P3 *  *  *  *  *  *  *  *  * \n"
			   "P4 *  *  *  *  *  * +FU *  * \n"
			   "P5 *  *  *  *  *  *  *  *  * \n"
			   "P6 *  *  *  *  *  *  *  *  * \n"
			   "P7 *  *  *  *  *  *  *  *  * \n"
			   "P8 *  *  *  *  *  *  *  *  * \n"
			   "P9 *  * +OU *  *  *  *  *  * \n"
			   "P+00KI00KI\n"
			   "P-00AL\n"
			   "+\n").getInitialState());
    FixedDepthSearcher searcher(state);
    Move best_move;
    ProofDisproof pdp=searcher.hasCheckmateMove<BLACK>(3, best_move);
    CPPUNIT_ASSERT_EQUAL(ProofDisproof::Checkmate(), pdp);
    CPPUNIT_ASSERT_EQUAL(Move(Position(3,3),GOLD,BLACK), best_move);
  }  
}

void CheckmateFixedDepthSearcherTest::testProofPieces()
{
  {
    NumEffectState state(CsaString(
			   "P1 *  *  *  *  *  *  *  *  * \n"
			   "P2 *  *  *  *  *  * -OU *  * \n"
			   "P3 *  *  *  *  *  *  *  *  * \n"
			   "P4 *  *  *  *  *  * +FU *  * \n"
			   "P5 *  *  *  *  *  *  *  *  * \n"
			   "P6 *  *  *  *  *  *  *  *  * \n"
			   "P7 *  *  *  *  *  *  *  *  * \n"
			   "P8 *  *  *  *  *  *  *  *  * \n"
			   "P9 *  * +OU *  *  *  *  *  * \n"
			   "P+00KI00KI\n"
			   "P-00AL\n"
			   "+\n").getInitialState());
    FixedDepthSearcher searcher(state);
    Move best_move;
    PieceStand pieces;
    ProofDisproof pdp=searcher.hasCheckmateMove<BLACK>(3, best_move, pieces);
    CPPUNIT_ASSERT_EQUAL(ProofDisproof::Checkmate(), pdp);
    CPPUNIT_ASSERT_EQUAL(Move(Position(3,3),GOLD,BLACK), best_move);
    CPPUNIT_ASSERT_EQUAL(PieceStand(0,0,0,0,2,0,0,0), pieces);
  }  
  {
    NumEffectState state(CsaString(
			   "P1 *  *  *  *  *  *  * -OU * \n"
			   "P2 *  *  *  *  *  * -FU-FU-FU\n"
			   "P3 *  *  *  *  *  *  *  *  * \n"
			   "P4 *  *  *  *  * +RY *  *  * \n"
			   "P5 *  *  *  *  *  *  *  *  * \n"
			   "P6 *  *  *  *  *  *  *  *  * \n"
			   "P7 *  *  *  *  *  *  *  *  * \n"
			   "P8+KI+KI+KI+KI *  *  *  *  * \n"
			   "P9+HI * +OU *  *  *  *  *  * \n"
			   "P-00FU\n"
			   "P+00AL\n"
			   "+\n").getInitialState());
    FixedDepthSearcher searcher(state);
    Move best_move;
    PieceStand pieces;
    ProofDisproof pdp=searcher.hasCheckmateMove<BLACK>(2, best_move, pieces);
    CPPUNIT_ASSERT_EQUAL(ProofDisproof::Checkmate(), pdp);
    CPPUNIT_ASSERT_EQUAL(Move(Position(4,4),Position(4,1),PROOK,PTYPE_EMPTY,
			      false,BLACK), 
			 best_move);
    CPPUNIT_ASSERT_EQUAL(PieceStand(0,4,4,4,0,2,0,0), pieces);
  }  
  {
    NumEffectState state(CsaString(
			   "P1 *  *  *  *  *  *  *  *  * \n"
			   "P2 *  *  *  *  *  *  * -OU-GI\n"
			   "P3 *  *  *  *  *  *  *  *  * \n"
			   "P4 *  *  *  *  *  * -FU-KE-FU\n"
			   "P5 *  *  *  *  * +KE *  *  * \n"
			   "P6+KA *  *  *  *  *  *  *  * \n"
			   "P7+KA *  *  *  *  *  *  *  * \n"
			   "P8+GI+GI+GI *  *  *  *  *  * \n"
			   "P9+HI+OU *  *  *  *  *  *  * \n"
			   "P-00FU\n"
			   "P+00AL\n"
			   "+\n").getInitialState());
    FixedDepthSearcher searcher(state);
    Move best_move;
    PieceStand pieces;
    ProofDisproof pdp=searcher.hasCheckmateMove<BLACK>(3, best_move, pieces);
    CPPUNIT_ASSERT_EQUAL(ProofDisproof::Checkmate(), pdp);
    CPPUNIT_ASSERT_EQUAL(Move(Position(3,3),GOLD,BLACK), best_move);
    CPPUNIT_ASSERT_EQUAL(PieceStand(0,0,1,0,2,0,0,0), pieces);
  }  
}

void CheckmateFixedDepthSearcherTest::testSuicide()
{
  {
    // ƨĤ꤬ƨƤʤʤ
    NumEffectState state(CsaString(
			   "P1 *  *  *  *  *  * +KI+KI-OU\n"
			   "P2 *  *  *  *  *  *  * -KI * \n"
			   "P3 *  *  *  *  *  *  *  * +FU\n"
			   "P4 *  *  *  *  * +KA *  *  * \n"
			   "P5 *  *  *  *  *  *  *  *  * \n"
			   "P6 *  *  *  *  *  *  *  *  * \n"
			   "P7 *  *  *  *  *  *  *  *  * \n"
			   "P8 *  *  *  *  *  *  *  *  * \n"
			   "P9 *  * +OU *  *  *  *  *  * \n"
			   "P+00FU00FU\n"
			   "P-00AL\n"
			   "-\n").getInitialState());
    FixedDepthSearcher searcher(state);
    Move lastMove=Move(Position(2,1),GOLD,BLACK);
    ProofDisproof pdp=searcher.hasEscapeMove<BLACK>(lastMove,/*0*/1);
    CPPUNIT_ASSERT_EQUAL(ProofDisproof::NoEscape(),pdp);
  }
  {
    // ƨĤ꤬ƨƤʤʤ
    NumEffectState state(CsaString(
			   "P1 *  *  *  *  * +KI+HI-OU * \n"
			   "P2 *  *  *  *  *  * -FU-FU-FU\n"
			   "P3 *  *  *  *  *  *  *  *  * \n"
			   "P4 *  *  *  *  *  *  *  *  * \n"
			   "P5 *  *  *  *  *  *  *  *  * \n"
			   "P6 *  *  *  *  *  *  *  *  * \n"
			   "P7 *  *  *  *  *  *  *  *  * \n"
			   "P8 *  *  *  *  *  *  *  *  * \n"
			   "P9 *  * +OU *  *  *  *  *  * \n"
			   "P+00FU00FU\n"
			   "P-00AL\n"
			   "-\n").getInitialState());
    FixedDepthSearcher searcher(state);
    Move lastMove=Move(Position(3,1),ROOK,BLACK);
    ProofDisproof pdp=searcher.hasEscapeMove<BLACK>(lastMove,0);
    CPPUNIT_ASSERT_EQUAL(ProofDisproof::NoEscape(),pdp);
  }
  {
    // openʼ
    NumEffectState state(CsaString(
			   "P1 *  *  *  *  *  *  * -OU-KA\n"
			   "P2 *  *  *  *  *  *  *  *  * \n"
			   "P3 *  *  *  *  *  * +KI *  * \n"
			   "P4 *  *  *  *  * +OU *  *  * \n"
			   "P5 *  *  *  *  *  *  *  *  * \n"
			   "P6 *  *  *  *  *  *  *  *  * \n"
			   "P7 *  *  *  *  *  *  *  *  * \n"
			   "P8 *  *  *  *  *  *  *  *  * \n"
			   "P9 *  *  *  *  *  *  *  *  * \n"
			   "P-00AL\n"
			   "+\n").getInitialState());
    FixedDepthSearcher searcher(state);
    Move best_move;
    ProofDisproof pdp=searcher.hasCheckmateMove<BLACK>(0, best_move);
    CPPUNIT_ASSERT(!pdp.isCheckmateSuccess() && !pdp.isCheckmateFail());
    pdp=searcher.hasCheckmateMove<BLACK>(1, best_move);
    CPPUNIT_ASSERT(!pdp.isCheckmateSuccess() && !pdp.isCheckmateFail());
  }
  {
    // ư
    NumEffectState state(CsaString(
			   "P1 *  *  * -KY * -KY * -OU * \n"
			   "P2 *  *  *  *  *  *  *  *  * \n"
			   "P3 *  *  *  *  *  *  *  *  * \n"
			   "P4 *  *  *  * +OU *  *  *  * \n"
			   "P5 *  *  * +KA *  *  *  *  * \n"
			   "P6 *  *  *  *  *  *  *  *  * \n"
			   "P7 *  *  *  *  *  *  *  *  * \n"
			   "P8 *  *  *  *  *  *  *  *  * \n"
			   "P9 *  *  *  *  *  *  *  *  * \n"
			   "P-00AL\n"
			   "+\n").getInitialState());
    FixedDepthSearcher searcher(state);
    Move best_move;
    ProofDisproof pdp=searcher.hasCheckmateMove<BLACK>(0, best_move);
    CPPUNIT_ASSERT(!pdp.isCheckmateSuccess() && !pdp.isCheckmateFail());
    pdp=searcher.hasCheckmateMove<BLACK>(1, best_move);
    CPPUNIT_ASSERT(!pdp.isCheckmateSuccess() && !pdp.isCheckmateFail());
  }
}

void CheckmateFixedDepthSearcherTest::testCheckInCheck()
{
  {
    NumEffectState state(CsaString(
			   "P1 *  *  *  * -OU *  *  *  * \n"
			   "P2 *  *  *  *  * +GI *  *  * \n"
			   "P3 *  *  *  *  *  *  *  *  * \n"
			   "P4 *  *  *  *  *  *  *  *  * \n"
			   "P5 *  *  *  *  *  *  *  *  * \n"
			   "P6-HI-HI-KA-KA *  *  *  * +KY\n"
			   "P7-FU-GI * +GI *  *  *  * +KY\n"
			   "P8 * -FU *  *  *  *  *  * +KY\n"
			   "P9+OU * +FU+FU+FU+FU+FU * +KY\n"
			   "P+00KI00KI00GI00FU00FU00FU00FU00FU00FU00FU00FU00FU00FU00FU\n"
			   "P-00KI00KI00KE00KE00KE00KE\n"
			   "-\n").getInitialState());
    // 98ΰͤǤϤ뤱
    FixedDepthSearcher searcher(state);
    Move best_move;
    ProofDisproof pdp=searcher.hasCheckmateMove<WHITE>(0, best_move);
    CPPUNIT_ASSERT(!pdp.isCheckmateSuccess() && !pdp.isCheckmateFail());

    pdp=searcher.hasCheckmateMove<WHITE>(1, best_move);
    CPPUNIT_ASSERT(pdp.isCheckmateFail());
  }
}

int checkCount;
int checkCounts[20];
static void testCheckFile(std::string filename)
{
  filename = OslConfig::testFile(filename);
  
  checkCount++;
  Record rec=CsaFile(filename).getRecord();
  vector<osl::Move> dummy=rec.getMoves();
  SimpleState sstate=rec.getInitialState();

  NumEffectState state(sstate);
  FixedDepthSearcher searcher(state);
  Move best_move;
  ProofDisproof pdp;
  for(int i=0;i<=8;i++){
    if(state.getTurn()==BLACK)
      pdp=searcher.hasCheckmateMove<BLACK>(i, best_move);
    else
      pdp=searcher.hasCheckmateMove<WHITE>(i, best_move);
    CPPUNIT_ASSERT(!pdp.isCheckmateFail() ||
		   (std::cerr << std::endl << best_move << std::endl <<
		    state << std::endl,0));
    if(pdp.isCheckmateSuccess()){
      checkCounts[i]++;
    }
  }
}
void CheckmateFixedDepthSearcherTest::testCheckFiles()
{
  std::ifstream ifs(OslConfig::testFile("kisenCheckProblems/FILES"));
  CPPUNIT_ASSERT(ifs);
  int i=0;
  int count=500;
  if (isShortTest)
    count=2;
  std::string filename;
  while((ifs >> filename) && (++i<count)) {
    if(filename == "") 
      break;
    if (! isShortTest)
      std::cerr << "testCheckFile: " << filename << " ";
    testCheckFile("kisenCheckProblems/" + filename);
  }
  if(!isShortTest){
    std::cerr << std::endl << "checkCount=" << checkCount << std::endl;
    for(i=0;i<=8;i++){
      std::cerr  << "checkCounts[" << i << "]=" << checkCounts[i] << std::endl;
    }
  }
}

int noCheckCount;
int noCheckCounts[20];
static void testNoCheckFile(std::string filename)
{
  filename = OslConfig::testFile(filename);

  noCheckCount++;
  Record rec=CsaFile(filename).getRecord();
  vector<osl::Move> dummy=rec.getMoves();
  SimpleState sstate=rec.getInitialState();

  NumEffectState state(sstate);
  FixedDepthSearcher searcher(state);
  Move best_move;
  ProofDisproof pdp;
  for(int i=0;i<=8;i++){
    if(state.getTurn()==BLACK)
      pdp=searcher.hasCheckmateMove<BLACK>(i, best_move);
    else
      pdp=searcher.hasCheckmateMove<WHITE>(i, best_move);
    CPPUNIT_ASSERT(!pdp.isCheckmateSuccess() ||
		   (std::cerr << std::endl << best_move << std::endl <<
		    state << std::endl,0));
    if(pdp.isCheckmateFail()){
      noCheckCounts[i]++;
    }
  }
}
void CheckmateFixedDepthSearcherTest::testNoCheckFiles()
{
  std::ifstream ifs(OslConfig::testFile("kisenNoCheckProblems/FILES"));
  CPPUNIT_ASSERT(ifs);
  int i=0;
  int count=500;
  if (isShortTest)
    count=2;
  std::string filename;
  while((ifs >> filename) && (++i<count)) {
    if(filename == "") 
      break;
    if (! isShortTest)
      std::cerr << "testNoCheckFile: " << filename << " ";
    testNoCheckFile("kisenNoCheckProblems/" + filename);
  }
  if(!isShortTest){
    std::cerr << std::endl << "noCheckCount=" << noCheckCount << std::endl;
    for(i=0;i<=8;i++){
      std::cerr  << "noCheckCounts[" << i << "]=" << noCheckCounts[i] << std::endl;
    }
  }
}

void CheckmateFixedDepthSearcherTest::testNoCheckmate()
{
  {
    NumEffectState state(CsaString(
			   "P1-KY * -HI-FU *  * -KA-KE-KY\n"
			   "P2 *  *  *  * +GI-OU-KI *  * \n"
			   "P3 *  *  * +TO * -KI-GI-FU * \n"
			   "P4-FU * -FU * -FU-FU-FU * -FU\n"
			   "P5 *  *  * +GI *  *  * +FU+FU\n"
			   "P6+FU * +FU * +FU+GI *  *  * \n"
			   "P7 * +FU * +KI * +FU *  *  * \n"
			   "P8 *  * +KI+KA *  *  * +HI * \n"
			   "P9+KY+KE+OU *  *  *  * +KE+KY\n"
			   "P+00KE00FU\n"
			   "P-00FU\n"
			   "+\n").getInitialState());
    FixedDepthSearcher searcher(state);
    Move best_move;
    ProofDisproof pdp=searcher.hasCheckmateMove<BLACK>(0, best_move);
    CPPUNIT_ASSERT(!pdp.isCheckmateSuccess());

    // r3738ޤǤ+6886KAǵͤߤȸǧƤ
    pdp=searcher.hasCheckmateMove<BLACK>(2, best_move);
    CPPUNIT_ASSERT(!pdp.isCheckmateSuccess());
  }
  {
    NumEffectState state(CsaString(
			   "P1-KY-KE *  *  *  *  * -KE-KY\n"
			   "P2 *  *  *  *  *  * +NK+KI * \n"
			   "P3 *  * -KA-FU-GI-KI * -FU-OU\n"
			   "P4-FU * -FU * -HI * -FU * -FU\n"
			   "P5 * -FU * +FU-FU *  * +FU * \n"
			   "P6+FU * +FU * +GI+FU+FU * +FU\n"
			   "P7 * +FU+GI+KI *  *  *  *  * \n"
			   "P8 * +OU+KI+KA *  *  * +HI * \n"
			   "P9+KY+KE *  *  *  *  *  * +KY\n"
			   "P+00GI00FU00FU\n"
			   "+\n").getInitialState());
    FixedDepthSearcher searcher(state);
    Move best_move;
    ProofDisproof pdp=searcher.hasCheckmateMove<BLACK>(0, best_move);
    CPPUNIT_ASSERT(!pdp.isCheckmateSuccess());

    pdp=searcher.hasCheckmateMove<BLACK>(2, best_move); // +4645FU-3435FU-2324FUԵ
    CPPUNIT_ASSERT(!pdp.isCheckmateSuccess());
  }
  {
    NumEffectState state(CsaString(
			   "P1 * -KE *  *  *  * -OU * -KY\n"
			   "P2+NG *  * -GI *  * -KI *  * \n"
			   "P3-KY * -KA-FU * -KI+NK * -FU\n"
			   "P4 * -FU-FU * -FU-FU-FU-FU * \n"
			   "P5-FU *  * +FU *  *  *  * +FU\n"
			   "P6 *  * +FU * +FU * +FU+FU * \n"
			   "P7+FU+FU+GI+KI * +FU *  *  * \n"
			   "P8 * +OU+KI *  * +GI * +HI * \n"
			   "P9+KY+KE *  *  *  *  *  * +KY\n"
			   "P+00HI00KA00KE\n"
			   "+\n").getInitialState());
    FixedDepthSearcher searcher(state);
    Move best_move;
    ProofDisproof pdp=searcher.hasCheckmateMove<BLACK>(0, best_move);
    CPPUNIT_ASSERT(!pdp.isCheckmateSuccess());

    // r3636ޤǤ+0061HIǵͤߤȸǧƤ
    pdp=searcher.hasCheckmateMove<BLACK>(2, best_move);
    CPPUNIT_ASSERT(!pdp.isCheckmateSuccess());
  }
}

// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
