/* effectUtil.t.cc
 */
#include "osl/effect_util/effectUtil.h"
#include "osl/effect_util/effectUtil.tcc"
#include "osl/state/numEffectState.h"
#include "osl/record/csaString.h"
#include "osl/misc/fixedCapacityVector.h"

#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>
#include <iostream>

class EffectUtilTest : public CppUnit::TestFixture 
{
  CPPUNIT_TEST_SUITE(EffectUtilTest);
  CPPUNIT_TEST(testMostValuableEffectOfMoveNotKing);
  CPPUNIT_TEST(testForEachEffectOfMove);
  CPPUNIT_TEST(testCheapest);
  CPPUNIT_TEST(testCheapestPtype);
  CPPUNIT_TEST(testPromotable);
  CPPUNIT_TEST(testSafeCaptureNotByKing);
  CPPUNIT_TEST(testIsOpenAttackMove);
  CPPUNIT_TEST(testIsAddEffectMove);
  CPPUNIT_TEST_SUITE_END();
public:
  void testMostValuableEffectOfMoveNotKing();
  void testCheapest();
  void testCheapestPtype();
  void testPromotable();
  void testForEachEffectOfMove();
  void testSafeCaptureNotByKing();
  void testIsOpenAttackMove();
  void testIsAddEffectMove();
};

CPPUNIT_TEST_SUITE_REGISTRATION(EffectUtilTest);

using namespace osl;

struct PositionVector : public FixedCapacityVector<Position,81>
{
  
};

std::ostream& operator<<(std::ostream& os, const PositionVector& vec)
{
  for (size_t i=0; i<vec.size(); ++i)
  {
    os << vec[i] << "\n";
  }
  return os;
}

struct StorePosition
{
  PositionVector& out;
  StorePosition(PositionVector& o) : out(o)
  {
  }
  void operator()(Position pos)
  {
    out.push_back(pos);
  }
};

void EffectUtilTest::testForEachEffectOfMove()
{
  const NumEffectState state(CsaString(
			       "P1-KY-KE-GI-KI-OU-KI-GI-KE-KY\n"
			       "P2 * -HI *  *  *  *  * -KA * \n"
			       "P3-FU-FU-FU-FU-FU-FU-FU-FU-FU\n"
			       "P4 *  *  *  *  *  *  *  *  * \n"
			       "P5 *  *  *  *  *  *  *  *  * \n"
			       "P6 *  *  *  *  *  *  *  *  * \n"
			       "P7+FU+FU+FU+FU+FU+FU+FU+FU+FU\n"
			       "P8 * +KA *  *  *  *  * +HI * \n"
			       "P9+KY+KE+GI+KI+OU+KI+GI+KE+KY\n"
			       "+\n").getInitialState());
  PositionVector result;
  StorePosition store(result);
  const Move m2726 = Move(Position(2,7),Position(2,6),
			     PAWN,PTYPE_EMPTY,false,BLACK);
  EffectUtil::forEachEffectPositionOfMove(state, m2726, store);
  CPPUNIT_ASSERT_EQUAL((size_t)1, result.size());
  CPPUNIT_ASSERT_EQUAL(Position(2,5), result[0]);
  
  // validMove ǤʤƤưΤǼȴ
  const Move m3334 = Move(Position(3,3),Position(3,4),
			     PAWN,PTYPE_EMPTY,false,WHITE);
  result.clear();
  EffectUtil::forEachEffectPositionOfMove(state, m3334, store);
  CPPUNIT_ASSERT_EQUAL((size_t)1, result.size());
  CPPUNIT_ASSERT_EQUAL(Position(3,5), result[0]);

  const Move m55gin = Move(Position(5,5), SILVER, BLACK);
  result.clear();
  EffectUtil::forEachEffectPositionOfMove(state, m55gin, store);
  CPPUNIT_ASSERT_EQUAL((size_t)5, result.size());

  const Move m15ryu = Move(Position(1,4), Position(1,5), 
			      PROOK, PTYPE_EMPTY, false, WHITE);
  result.clear();
  EffectUtil::forEachEffectPositionOfMove(state, m15ryu, store);
  CPPUNIT_ASSERT_EQUAL((size_t)14, result.size());

  const Move m37ke = Move(Position(2,9), Position(3,7), 
			     KNIGHT, PTYPE_EMPTY, false, BLACK);
  result.clear();
  EffectUtil::forEachEffectPositionOfMove(state, m37ke, store);
  CPPUNIT_ASSERT_EQUAL((size_t)2, result.size());
  CPPUNIT_ASSERT_EQUAL(Position(4,5), result[0]);
  CPPUNIT_ASSERT_EQUAL(Position(2,5), result[1]);
}

void EffectUtilTest::testMostValuableEffectOfMoveNotKing()
{
  const char *endGameState =
  "P1+NY * +TO *  *  * -OU-KE-KY\n"
  "P2 *  *  *  *  * -GI-KI *  *\n"
  "P3 * +RY *  * +UM * -KI-FU-FU\n"
  "P4 *  * +FU-FU *  *  *  *  *\n"
  "P5 *  * -KE * +FU *  * +FU *\n"
  "P6-KE *  * +FU+GI-FU *  * +FU\n"
  "P7 *  * -UM *  *  *  *  *  *\n"
  "P8 *  *  *  *  *  *  *  *  * \n"
  "P9 * +OU * -GI *  *  *  * -NG\n"
  "P+00HI00KI00KE00KY00FU00FU00FU00FU00FU00FU00FU\n"
  "P-00KI00KY00FU\n"
  "P-00AL\n"
  "+\n";
  const NumEffectState state(CsaString(endGameState).getInitialState());
  const Move m64um = Move(Position(5,3), Position(5,4), 
			     PBISHOP, PTYPE_EMPTY, false, BLACK);
  {
    const Piece highestB 
      = EffectUtil::mostValuableEffectOfMoveNotKing(BLACK, state, m64um);
    CPPUNIT_ASSERT_EQUAL(state.getPieceAt(Position(5,5)), highestB);
    const Piece highestW
      = EffectUtil::mostValuableEffectOfMoveNotKing(WHITE, state, m64um);
    CPPUNIT_ASSERT_EQUAL(state.getPieceAt(Position(3,2)), highestW);
  }   
  const Move m78um = Move(Position(7,7), Position(7,8), 
			     PBISHOP, PTYPE_EMPTY, false, BLACK);
  {
    const Piece highestB 
      = EffectUtil::mostValuableEffectOfMoveNotKing(BLACK, state, m78um);
    CPPUNIT_ASSERT_EQUAL(state.getPieceAt(Position(5,6)), highestB);
    const Piece highestW
      = EffectUtil::mostValuableEffectOfMoveNotKing(WHITE, state, m78um);
    CPPUNIT_ASSERT_EQUAL(state.getPieceAt(Position(6,9)), highestW);
  }   
}


void EffectUtilTest::testCheapest()
{
  const char *endGameState =
  "P1+NY * +TO *  *  * -OU-KE-KY\n"
  "P2 *  *  *  *  * -GI-KI *  *\n"
  "P3 * +RY *  * +UM * -KI-FU-FU\n"
  "P4 *  * +FU-FU *  *  *  *  *\n"
  "P5 *  * -KE * +FU *  * +FU *\n"
  "P6-KE *  * +FU+GI-FU *  * +FU\n"
  "P7 *  * -UM *  *  *  *  *  *\n"
  "P8 *  *  *  *  *  *  *  *  * \n"
  "P9 * +OU * -GI *  *  *  * -NG\n"
  "P+00HI00KI00KE00KY00FU00FU00FU00FU00FU00FU00FU\n"
  "P-00KI00KY00FU\n"
  "P-00AL\n"
  "+\n";
  const NumEffectState state(CsaString(endGameState).getInitialState());
  CPPUNIT_ASSERT_EQUAL(state.getPieceOnBoard(Position(8,3)),
		       EffectUtil::cheapestPiece
		       (BLACK, state, Position(9,3)));
  CPPUNIT_ASSERT_EQUAL(state.getPieceOnBoard(Position(9,6)),
		       EffectUtil::cheapestPiece
		       (WHITE, state, Position(8,8)));
  CPPUNIT_ASSERT_EQUAL(Piece::EMPTY(),
		       EffectUtil::cheapestPiece
		       (BLACK, state, Position(2,8)));
  CPPUNIT_ASSERT_EQUAL(state.getPieceOnBoard(Position(3,2)),
		       EffectUtil::cheapestPiece
		       (WHITE, state, Position(2,2)));
}

void EffectUtilTest::testCheapestPtype()
{
  const char *endGameState =
  "P1+NY * +TO *  *  * -OU-KE-KY\n"
  "P2 *  *  *  *  * -GI-KI *  *\n"
  "P3 * +RY *  * +UM * -KI-FU-FU\n"
  "P4 *  * +FU-FU *  *  *  *  *\n"
  "P5 *  * -KE * +FU *  * +FU *\n"
  "P6-KE *  * +FU+GI-FU *  * +FU\n"
  "P7 *  * -UM *  *  *  *  *  *\n"
  "P8 *  *  *  *  *  *  *  *  * \n"
  "P9 * +OU * -GI *  *  *  * -NG\n"
  "P+00HI00KI00KE00KY00FU00FU00FU00FU00FU00FU00FU\n"
  "P-00KI00KY00FU\n"
  "P-00AL\n"
  "+\n";
  const NumEffectState state(CsaString(endGameState).getInitialState());
  CPPUNIT_ASSERT_EQUAL(PROOK, EffectUtil::cheapestPtype(BLACK, state, Position(9,3)));
  CPPUNIT_ASSERT_EQUAL(KNIGHT, EffectUtil::cheapestPtype(WHITE, state, Position(8,8)));
  CPPUNIT_ASSERT_EQUAL(PTYPE_EMPTY, EffectUtil::cheapestPtype(BLACK, state, Position(2,8)));
  CPPUNIT_ASSERT_EQUAL(GOLD, EffectUtil::cheapestPtype(WHITE, state, Position(2,2)));
}

void EffectUtilTest::testPromotable()
{
  const char *endGameState =
  "P1+NY * +TO *  *  * -OU-KE-KY\n"
  "P2 *  *  *  *  * -GI-KI *  *\n"
  "P3 * +RY *  * +UM * -KI-FU-FU\n"
  "P4 *  * +FU-FU *  *  *  *  *\n"
  "P5 *  * -KE * +FU *  * +FU *\n"
  "P6-KE *  * +FU+GI-FU *  * +FU\n"
  "P7 *  * -UM *  *  *  *  *  *\n"
  "P8 *  *  *  *  *  *  *  *  * \n"
  "P9 * +OU * -GI *  *  *  * -NG\n"
  "P+00HI00KI00KE00KY00FU00FU00FU00FU00FU00FU00FU\n"
  "P-00KI00KY00FU\n"
  "P-00AL\n"
  "+\n";
  const NumEffectState state(CsaString(endGameState).getInitialState());
  CPPUNIT_ASSERT_EQUAL(Piece::EMPTY(),
		       EffectUtil::promotablePiece
		       (BLACK, state, Position(9,3)));
  CPPUNIT_ASSERT_EQUAL(state.getPieceOnBoard(Position(9,6)),
		       EffectUtil::promotablePiece
		       (WHITE, state, Position(8,8)));
  CPPUNIT_ASSERT_EQUAL(state.getPieceOnBoard(Position(7,4)),
		       EffectUtil::promotablePiece
		       (BLACK, state, Position(7,3)));
  CPPUNIT_ASSERT_EQUAL(state.getPieceOnBoard(Position(7,5)),
		       EffectUtil::promotablePiece
		       (WHITE, state, Position(6,7)));
}

void EffectUtilTest::testSafeCaptureNotByKing()
{
  const char *state_false_string =
    "P1-OU-KI * +HI *  *  *  *  * \n"
    "P2+FU *  *  *  *  *  *  *  * \n"
    "P3 * +TO *  *  *  *  *  *  * \n"
    "P4 *  *  *  *  *  *  *  *  * \n"
    "P5 *  *  *  *  *  *  *  *  * \n"
    "P6 *  *  *  *  *  *  *  *  * \n"
    "P7 *  *  *  *  *  *  *  *  * \n"
    "P8 *  *  *  *  *  *  *  *  * \n"
    "P9 *  *  *  *  *  *  *  * +OU\n"
    "P+00FU\n"
    "P-00AL\n"
    "-\n";
  const NumEffectState state_false(CsaString(state_false_string).getInitialState());
  const Position target = Position(9,2);
  CPPUNIT_ASSERT_EQUAL(Piece::EMPTY(), EffectUtil::safeCaptureNotByKing
		       (WHITE, state_false, target));
  
  const char *state_true_string =
    "P1-OU-KI * +KA *  *  *  *  * \n"
    "P2+FU *  *  *  *  *  *  *  * \n"
    "P3 * +TO *  *  *  *  *  *  * \n"
    "P4 *  *  *  *  *  *  *  *  * \n"
    "P5 *  *  *  *  *  *  *  *  * \n"
    "P6 *  *  *  *  *  *  *  *  * \n"
    "P7 *  *  *  *  *  *  *  *  * \n"
    "P8 *  *  *  *  *  *  *  *  * \n"
    "P9 *  *  *  *  *  *  *  * +OU\n"
    "P+00FU\n"
    "P-00AL\n"
    "-\n";
  const NumEffectState state_true(CsaString(state_true_string).getInitialState());
  CPPUNIT_ASSERT_EQUAL(state_true.getPieceOnBoard(Position(8,1)), 
		       EffectUtil::safeCaptureNotByKing
		       (WHITE, state_true, target));
}

void EffectUtilTest::testIsOpenAttackMove()
{
  {
    const char *state_string =
      "P1-KY+HI * +KA *  *  * -KE-KY\n"
      "P2 *  * -KI *  *  *  *  *  * \n"
      "P3-KE-KI-OU * +UM-GI-FU+FU-FU\n"
      "P4 * -GI *  * -FU *  * -FU * \n"
      "P5-FU *  *  *  * +GI *  *  * \n"
      "P6 * +FU-FU *  *  *  *  *  * \n"
      "P7+FU *  *  *  *  * +FU * +FU\n"
      "P8+KY+KI+KI *  *  *  *  *  * \n"
      "P9+OU+KE *  *  *  *  * +KE+KY\n"
      "P+00FU\n"
      "P-00HI00GI00FU00FU00FU00FU00FU00FU\n"
      "+\n";
    const NumEffectState state(CsaString(state_string).getInitialState());
    CPPUNIT_ASSERT(EffectUtil::isOpenAttackMove<BLACK>(state,Position(2,1),Position(6,1),Position(7,2)));
    // direct attack
    CPPUNIT_ASSERT(!EffectUtil::isOpenAttackMove<BLACK>(state,Position(6,3),Position(6,1),Position(7,2)));
    // not open
    CPPUNIT_ASSERT(!EffectUtil::isOpenAttackMove<BLACK>(state,Position(9,5),Position(9,7),Position(9,6)));
  }
}

void EffectUtilTest::testIsAddEffectMove()
{
  {
    const char *state_string =
      "P1-KY+HI * +KA *  *  * -KE-KY\n"
      "P2 *  * -KI *  *  *  *  *  * \n"
      "P3-KE-KI-OU * +UM-GI-FU+FU-FU\n"
      "P4 * -GI *  * -FU *  * -FU * \n"
      "P5-FU *  *  *  * +GI *  *  * \n"
      "P6 * +FU-FU *  *  *  *  *  * \n"
      "P7+FU *  *  *  *  * +FU * +FU\n"
      "P8+KY+KI+KI *  *  *  *  *  * \n"
      "P9+OU+KE *  *  *  *  * +KE+KY\n"
      "P+00FU\n"
      "P-00HI00GI00FU00FU00FU00FU00FU00FU\n"
      "-\n";
    const NumEffectState state(CsaString(state_string).getInitialState());
    CPPUNIT_ASSERT(EffectUtil::isAddEffectMove<WHITE>(state,Position(6,1),Move(Position(7,2),Position(7,1),GOLD,PTYPE_EMPTY,false,WHITE)));
    CPPUNIT_ASSERT(!EffectUtil::isAddEffectMove<WHITE>(state,Position(7,2),Move(Position(7,2),Position(6,3),GOLD,PTYPE_EMPTY,false,WHITE)));
    // open
    CPPUNIT_ASSERT(EffectUtil::isAddEffectMove<WHITE>(state,Position(9,4),Move(Position(9,3),Position(8,5),KNIGHT,PTYPE_EMPTY,false,WHITE)));
  }
}

/* ------------------------------------------------------------------------- */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
